"""Examples of how to compute the various quantities presented in the paper
"The Andreev rectifier: a nonlocal conductance signature of topological
phase transitions"
by Tómas Örn Rosdahl, Adriaan Vuik, Morten Kjærgaard and Anton Akhmerov.

arXiv:1706.08888."""

import kwant
import numpy as np
import matplotlib.pyplot as plt
import nonlocal_conductance as nlc
import constants as const
import tools as tls


def s_matrix(syst, energy, c):
    """Compute the scattering matrix at a given energy.

    Parameters:
    ------------
    syst: kwant.system.FiniteSystem object
        Finalized Kwant system of the scattering region with leads.
    E: float
        Energy at which to compute the scattering matrix.
    c: tools.SimpleNamespace object
        A container for all Hamiltonian parameters. """
    return kwant.smatrix(syst, energy, args = [c])


def cond_tunnel_gates(E, syst, c, lead_out, lead_in, sc_chains=True):
    """Compute various transport processes at a single energy in a setup
    with two normal leads.

    Parameters:
    ------------
    E: float
        Energy at which to compute the transport processes.
    syst: kwant.system.FiniteSystem object
        Finalized Kwant system of the scattering region with leads.
    c: tools.SimpleNamespace object
        A container for all Hamiltonian parameters.
    lead_out: integer
        Index of the out lead, to which transport takes place.
    lead_in: integer
        Index of the in lead, from which electrons are injected.
    sc_chains: bool
        Specify whether to treat the superconductor as a single lead, or
        as multiple leads (a sequence of decoupled chains).

    Returns:
    ------------
    nonloc: float
        The nonlocal conductance between the two leads, in units of e^2/h.
    tee: float
        Normal transmission, electron to electron.
    the: float
        Transmission corresponding to crossed Andreev reflection.
    loc: float
        The local conductance in the in lead, in units of e^2/h.
    ree: float
        Normal reflection, electron to electron.
    rhe: float
        Andreev reflection, electron to hole."""

    smatrix = s_matrix(syst, E, c)
    nonloc = nlc.get_cond(smatrix, lead_out, lead_in, trans = 'nonloc')
    tee = nlc.get_cond(smatrix, lead_out, lead_in, trans = 'tee')
    the = nlc.get_cond(smatrix, lead_out, lead_in, trans = 'the')
    loc = nlc.get_cond(smatrix, lead_in, lead_in, trans = 'loc')
    ree = nlc.get_cond(smatrix, lead_in, lead_in, trans = 'ree')
    rhe = nlc.get_cond(smatrix, lead_in, lead_in, trans = 'rhe')
    return nonloc, tee, the, loc, ree, rhe


def bands_lateral_sc(W_N, W_SC, c, klower = 0.0, kupper = np.pi,
                     kpoints = 401, scale=False):
    """Compute band structure of a ribbon that is partially superconducting and
    partially normal. Also returns the electron dispersion of the normal ribbon
    without the superconductor.

    Parameters:
    -------------
    W_N: float
        Width of the normal part in nm.
    W_SC: float
        Width of the superconducting part in nm.
    c: tools.SimpleNamespace object
        A container with all Hamiltonian parameters.
    klower: float
        Lower limit of the momentum interval, scaled in units of the lattice
        constant.
    kupper: float
        Upper limit of the momentum interval, scaled in units of the lattice
        constant.
    kpoints: int
        Number of discretization points in momentum.
    scale: bool
        Whether or not to scale the dispersion in terms of the bulk
        superconducting gap.

    Returns:
    --------------
    ks: numpy array
        Momentum values at which band structure was calculated.
    sc_disp: numpy array
        Dispersion of the normal-superconductor ribbon.
    norm_disp: numpy array
        Dispersion of the normal part only, without the superconductor.
        """
    ks = np.linspace(klower, kupper, kpoints)
    # Normal system
    norm_disp = np.array([nlc.get_bands_normal(k, c, W_N) for k in ks])
    # Lateral SC system
    sc_disp = np.array([nlc.get_bands(k, c, W_N, W_SC) for k in ks])
    if scale:
        sc_disp = sc_disp/c.delta
        norm_disp = norm_disp/c.delta
    return ks, sc_disp, norm_disp


def transport():
    """"An example of how to set up a proximitised system coupled to
    two normal leads and compute transport processes. """

    a = 10.0 # Lattice constant [nm]
    T = 1000*const.hbar**2/(2*const.m_eff_InAs*a**2*1e-18*const.e)   # meV

    # Compute the decay length of the slowest decaying mode
    W_L = 100.0 # nm
    dW_ML = 0.0 # nm
    sc_c = tls.SimpleNamespace(T=T, a=a, y_scale=1.0, x_scale=0.0, mu=3.0,
                               delta=const.delta_InAs, Ez=0,
                               alpha=const.alpha_InAs/a)
    W_SC = 1500-W_L-dW_ML
    decay_l = nlc.decay_length(W_L+dW_ML, W_SC, sc_c)  # Decay length in nm
    coeff = 0.5
    L = int(np.round(coeff*decay_l, -1))  # Length in nm

    # Dimension parameters
    dim_par = nlc.DimensionParameters(L=nlc.d_sys(L,a), W_L=nlc.d_sys(W_L,a),
                                      dW_ML = int(np.round(dW_ML/a)),
                                      dW_SC = int(np.round(10/a)),
                                      d=int(np.round(50/a)),
                                      dgT=int(np.round(20/a)),
                                      dL=int(np.round(250/a)),
                                      wgt=int(np.round(100/a)),
                                      dW_T=int(np.round(10/a)))

    # Container for parameters
    c = tls.SimpleNamespace(T=sc_c.T, a=sc_c.a, y_scale=sc_c.y_scale,
                            x_scale=sc_c.x_scale, alpha=sc_c.alpha,
                            Ez=sc_c.Ez, delta=sc_c.delta, mu=sc_c.mu,
                            mu_sc=sc_c.mu, dis=0.0, disorder=nlc.disorder,
                            salt='', tunnel_pots=nlc.tunnel_pots,
                            dim_par=dim_par)
    V_gtL = 0.0   # Left junction gate potentials, meV
    V_gtR = 0.0   # Right junction gate potentials, meV
    gate_par = nlc.GateParameters(V_gtL=V_gtL, V_gtR=V_gtR)
    c.gate_par = gate_par

    # Compute conductance as a function of bias
    start_bias = -0.35; end_bias = 0.35  # meV
    bias_vals = np.linspace(start_bias, end_bias, 101)
    lead_out = 1
    lead_in = 0
    sc_chains = True
    syst = nlc.make_NSN_tunnel_gates(c.dim_par, plot_sys=False,
                                     sc_chains=sc_chains)
    bias_cond = [cond_tunnel_gates(E, syst, c, lead_out, lead_in,
                                   sc_chains=sc_chains)
                 for E in bias_vals]
    # Extract the various processes
    nonloc, tee, the, loc, ree, rhe = zip(*bias_cond)
    plt.plot(bias_vals, nonloc)


def dispersion():
    """An example of how to compute dispersion relations. """

    a = 10.0  # Lattice constant [nm]
    T = 1000*const.hbar**2/(2*const.m_eff_InAs*a**2*1e-18*const.e)   # meV
    disp_c = tls.SimpleNamespace(T=T, a=a, x_scale=0.0, y_scale=1.0, mu=3,
                                 delta=const.delta_InAs, Ez=0,
                                 alpha=const.alpha_InAs/a)
    # Dimensions of the ribbon
    W_N = 100  # nm
    W_SC = 2500  # nm
    # Compute the dispersion
    ks, sc_disp, norm_disp = bands_lateral_sc(W_N, W_SC, disp_c, klower=-0.7,
                                              kupper=0.7, kpoints=101,
                                              scale=True)
    plt.plot(ks, sc_disp)
    plt.ylim([-1.2, 1.2])


def induced_gap():
    """An example of how to compute the induced gap. """

    a = 10.0  # Lattice constant [nm]
    T = 1000*const.hbar**2/(2*const.m_eff_InAs*a**2*1e-18*const.e)   # meV
    disp_c = tls.SimpleNamespace(T=T, a=a, x_scale=0.0, y_scale=1.0, mu=3,
                                 delta=const.delta_InAs, Ez=0,
                                 alpha=const.alpha_InAs/a)
    # Dimensions of the ribbon
    W_N = 100  # nm
    W_SC = 2500  # nm
    lead = nlc.make_inf_lateral_sys(nlc.d_sys(W_N, disp_c.a),
                                    nlc.d_sys(W_SC, disp_c.a))
    ind_gap = nlc.gap_finder(lead, disp_c, start_Es=(0, 2.0), eps=1e-10)
    return ind_gap


def density_of_states():
    """An example of how to compute the density of states. """

    a = 10.0  # Lattice constant [nm]
    T = 1000*const.hbar**2/(2*const.m_eff_InAs*a**2*1e-18*const.e)   # meV
    # Dimensions of the ribbon
    W_N = 100  # nm
    W_SC = 3000  # nm
    disp_c = tls.SimpleNamespace(T=T, a=a, x_scale=0.0, y_scale=1.0, mu=5.0,
                                 delta=const.delta_InAs, Ez=0, alpha=0)
    energies = np.linspace(0, 0.25, 101) # meV
    DOS = [nlc.DOS_parallel(E, disp_c, W_N, W_SC, a) for E in energies]
    out_data = (np.concatenate([-energies[::-1][:-1], energies]),
                 np.concatenate([DOS[::-1][:-1], DOS]))
    plt.plot(out_data[0]/disp_c.delta, out_data[1])
    plt.ylim([0, 70])