
# coding: utf-8

# # Generate plots from data and save as pdf

# In[1]:

try:
    get_ipython().magic('matplotlib inline')
except NameError:
    pass


# In[2]:

import sys, os
sys.path.append(os.path.dirname(os.getcwd()))

import numpy as np
import matplotlib
matplotlib.use('agg')
from scipy.integrate import simps


# In[3]:

import kwant
from kwant.digest import uniform

import nonlocal_conductance as nlc
import tools as tls
import constants as const


# In[4]:

import matplotlib.pyplot as plt
from matplotlib import ticker, gridspec
import matplotlib.gridspec as gridspec
import matplotlib.colors as colors
from mpl_toolkits.axes_grid1 import make_axes_locatable
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
from scipy import optimize, stats, signal
import pickle


# In[5]:

from matplotlib import rc
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
rc('text', usetex=True)
rc('axes', linewidth=1.5)


# In[6]:

kB = 8.6173324e-5 # eV/K


# In[7]:

class MidpointNormalize(colors.Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=0):
        self.midpoint = midpoint
        colors.Normalize.__init__(self, vmin, vmax)
    def __call__(self, value, clip=None):
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))
    
    
def latex(x, pos=None):
    return '${}$'.format(x)


def set_ticks(ax, xticks=(), yticks=(), logx=False, logy=False, xpower=False, ypower=False):
    """Add latex ticks to an Axes instance optionally setting log-scale."""
    if logx:
        ax.set_xscale('log')
        xfmt = '$10^{{{}}}$'
        xvals = [10**i for i in xticks]
    elif xpower:
        xfmt = '$10^{{{}}}$'
        xvals = xticks
    else:
        xfmt = '${}$'
        xvals = xticks

    if logy:
        ax.set_yscale('log')
        yfmt = '$10^{{{}}}$'
        yvals = [10**i for i in yticks]
    elif ypower:
        yfmt = '$10^{{{}}}$'
        yvals = yticks
    else:
        yfmt = '${}$'
        yvals = yticks

    if xticks is not None:
        ax.set_xticks(xvals)
        ax.set_xticklabels([xfmt.format(i) for i in xticks])

    if yticks is not None:
        ax.set_yticks(yvals)
        ax.set_yticklabels([yfmt.format(i) for i in yticks])
        
    
def read_for_waterfall(name = 'DATA'):
    with open(name + '.p', 'rb') as fp:
        data = pickle.load(fp)
        bias_vals = pickle.load(fp)
        keys = pickle.load(fp)
    return data, bias_vals, keys
        
    
def neg_fermi_der(E, mu, T):
    exponent = (E - mu)/(kB*T)
    return 1.0/(kB*T*(np.exp(exponent) + 1)*(np.exp(-exponent) + 1))

        
def temperature_cond_bias(energies, cond_trace, blims, T):
    """Broaden the conductance with temperature.
    
    Parameters:
    ---------------
    energies: numpy array of shape (n, )
        Energies at which the density of states has been calculated.
    cond_trace: numpy array of shape (n, )
        Conductance at given energy points.
    blims: list or tuple of two elements
        A subinterval of energies over which the broadened
        conductance is calculated.
    T: float
        Temperature in mK.
    
    Returns:
    ---------------
    bias_cond: numpy array
        The broadened conductance at all energy points in the interval
        determined by blims.
    bias_vals: numpy array
        Energy points at which the broadened conductance has been computed.
    """
    
    
    bias_min, bias_max = blims  # Energy limits for broadened conductance
    # Pick out bias values at which to evaluate conductance
    bias_vals = energies[energies >= bias_min]
    bias_vals = bias_vals[bias_vals <= bias_max]
    # Limits for the energy integral
    limit = 20*const.kB*T
    
    bias_cond = []
    for bias in bias_vals:
        # Integrate conductance over -limit + bias < E < bias + limit: limit is chosen
        # such that the Femri distribution is vanishing outside this interval
        inds = (energies >= bias - limit)*(energies <= bias + limit)
        integrand_energies = energies[inds]
        integrand_conds = cond_trace[inds]
        cond_ferm = np.array([neg_fermi_der(energy, bias, T)*cond for energy, cond in
                              zip(integrand_energies, integrand_conds)])
        bias_cond.append(simps(cond_ferm, integrand_energies))
    return np.array(bias_cond), bias_vals


def extract_cond_waterfall(data, bias_vals, keys, new_keys=None, plot='nonloc', trans='all'):
    new_data = {}
    for i, key in enumerate(reversed(keys)):
        if trans == 'all':
            nonloc, tee, the, loc, ree, rhe = zip(*data[key])
            data_dict = {'nonloc': nonloc, 'tee': tee, 'the': the, 'loc': loc, 'ree': ree, 'rhe': rhe}
        elif trans == 'nonloc':
            nonloc, tee, the = zip(*data[key])
            data_dict = {'nonloc': nonloc, 'tee': tee, 'the': the}
        elif trans == 'loc':
            loc, ree, rhe = zip(*data[key])
            data_dict = {'loc': loc, 'ree': ree, 'rhe': rhe}
        if new_keys:
            new_data[new_keys[i]] = np.array(data_dict[plot])
        else:
            new_data[key] = np.array(data_dict[plot])
    if new_keys:
        return new_data, bias_vals, new_keys
    else:
        return new_data, bias_vals, keys

    
def swap(pl):
    for dat in pl:
        xdata = dat.get_xdata()
        ydata = dat.get_ydata()
        dat.set_xdata(ydata)
        dat.set_ydata(xdata)
    return pl


def cond_waterfall(ax, data, bias_vals, keys, blims, flip=False, xticks = [-1, -0.5, 0, 0.5, 1],
                   xlim=None, ylim=None, delta=0.2, plot='nonloc', offset_step=0.0, T=None, title='',
                   yticks=[2, 3, 4], legend=True, legend_pos=(0.08, 1.4),
                   ncol=1, key_add=r'$L = $ ', textloc=[0.5, 0.8], sharexax=False, insert_title=False, 
                   yaxis_pos='left', right_yaxis=True, ylabel=r'$G_{21} \ [e^2/h]$', 
                   labelloc=[0.02, 0.8], label='$\mathrm{(a)}$', xlabel=r'$E/\Delta$',
                   ind_gap=None, shareyax=False):
    for j, key in enumerate(reversed(keys)):
        color = plt.cm.gist_heat(j/float(len(keys)))
        offset = keys.index(key)*offset_step
        if plot == 'nonloc':
            # Plot a dashed line at G = 0
            if not flip:
                ax.axhline(offset, c="black", linestyle='dashed')
            else:
                ax.axvline(offset, c="black", linestyle='dashed')
        # Plot a dotted line at E = delta and E = -delta, the bulk gap
        if not flip:
            ax.axvline(1.0, c="grey", linestyle='dashed')
            ax.axvline(-1.0, c="grey", linestyle='dashed')
            # Dashdotted line at the induced gap, if specified
            if ind_gap is not None:
                ax.axvline(ind_gap, c="grey", linestyle='dashdot')
                ax.axvline(-ind_gap, c="grey", linestyle='dashdot')
        else:
            ax.axhline(1.0, c="grey", linestyle='dashed')
            ax.axhline(-1.0, c="grey", linestyle='dashed')
            if ind_gap is not None:
                ax.axhline(ind_gap, c="grey", linestyle='dashdot')
                ax.axhline(-ind_gap, c="grey", linestyle='dashdot')
        # If no temperature specified, assume 0 temperature.
        if T is None:
            pl = ax.plot(np.array(bias_vals)/delta, np.array(data[key])+offset, label=key_add + key, linewidth=1.5, 
                         color=color)
        # Nonzero temperature specified.
        else:
            if blims is None:
                blims = [-delta-0.05, delta+0.05]
            temp_cond, temp_bias = temperature_cond_bias(bias_vals, np.array(data[key]), blims, T)
            pl = ax.plot(np.array(temp_bias)/delta, temp_cond+offset, label=key_add + key, linewidth=1.5, color=color)
        if flip:
            pl = swap(pl)
        if xlim is None:
            ax.set_xlim(np.array([-delta-0.05, delta+0.05])/delta)
        else:
            ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    #ax.set_title(title, fontsize=fontsize)
    ax.tick_params(width=1.5, size=6, labelsize=23)
    if yaxis_pos == 'right':
        if right_yaxis:
            ax.yaxis.tick_right()
        else:
            plt.setp(ax.get_yticklabels(), visible=False)
    elif yaxis_pos == 'left':
        ax.set_ylabel(ylabel, size=25)
    elif yaxis_pos == 'right_and_label':
        ax.yaxis.tick_right()
        ax.set_ylabel(ylabel, size=25)
        ax.yaxis.set_label_position("right")
    if insert_title:
        ax.text(textloc[0], textloc[1], title, transform=ax.transAxes, fontsize=25)
    ax.text(labelloc[0], labelloc[1], label, transform=ax.transAxes, fontsize=25)

    set_ticks(ax, xticks, yticks)
    
    # Place legend to the right of the figure
    if legend:
        ax.legend(bbox_to_anchor=legend_pos, ncol=ncol, loc=2, borderaxespad=0.)
        plt.setp(ax.get_legend().get_texts(), fontsize=25)    # the legend text fontsize
        
    if sharexax:
        plt.setp(ax.get_xticklabels(), visible=False)
    else:
        ax.set_xlabel(xlabel, size=25)
    if shareyax:
        plt.setp(ax.get_yticklabels(), visible=False)
        
    return ax


def lorentzian(x, x0, gamma):
    """A Lorentzian centered at x0 with full width at half maximum gamma,
    evaluated at the point x."""
    return gamma/( (x-x0)**2 + gamma**2 )/np.pi


def DOS_Lorentzian(energies, DOS, DElims, gamma):
    """Broaden the density of states by convoluting it with a Lorentzian.
    
    Parameters:
    ---------------
    energies: numpy array of shape (n, )
        Energies at which the density of states has been calculated.
    DOS: numpy array of shape (n, )
        Density of states at given energy points.
    DElims: list or tuple of two elements
        A subinterval of energies over which the broadened density
        of states is calculated.
    gamma: float
        Full width at half maximum for the Lorentzian.
    
    Returns:
    ---------------
    conv_DOS: numpy array
        The broadened density of states at all energy points in the interval
        determined by DElims.
    DE_vals: numpy array
        Energy points at which the broadened density of states has been computed.
    """
    
    DE_min, DE_max = DElims  # Energy limits for convoluted DOS
    # Pick out energy values at which to return convoluted DOS 
    DE_vals = energies[energies >= DE_min]
    DE_vals = DE_vals[DE_vals <= DE_max]
    
    conv_DOS = []
    for DE in DE_vals:
        DE_lor = np.array([lorentzian(energy, DE, gamma)*DOS_p for energy, DOS_p in zip(energies, DOS)])
        conv_DOS.append(simps(DE_lor, energies))
    return np.array(conv_DOS), DE_vals


# ## Scattering region wave functions for Fig. 2

# In[8]:

def plot_wf(syst, lead_num, energy, c, oversampling=1, method='nearest', colorbar=False, vmax=None, vmin=None,
           cmap=None, out_name='wf.pdf'):
    wf = kwant.solvers.default.wave_function(syst, energy=energy, args=(c,))
    wfs = wf(lead_num)
    # 4 orbitals per unit cell: sum over them. This is done in the second index of wfs.
    # The first index of wfs is over all incident modes. Only interested in injected electrons,
    # not holes, so we only sum over the first half of the incoming modes.
    fig = plt.figure()
    ax = plt.axes([1, 1, 1.5, 1.5]) # [left, bottom, width, height]
    wf = sum([np.abs(wfs[i,::4])**2 - np.abs(wfs[i,1::4])**2 + np.abs(wfs[i,2::4])**2 - np.abs(wfs[i,3::4])**2
              for i in range(wfs.shape[0]//2)])
    kwant.plotter.map(syst, wf, oversampling=oversampling, colorbar=colorbar, fig_size=[30,30], method=method,
                      vmin=vmin, vmax=vmax, cmap=cmap, ax=ax, show=False);
    ax.set_aspect(1.5)
    ax.axis('off')
    plt.savefig(out_name, bbox_inches='tight')
    

def scatter_wf():
    out_name = 'scatter_wf.pdf'
    # Set lattice constant a, energy unit and dimensions in units of a
    # Use meV in the Hamiltonian
    a = 10.0 # Lattice constant [nm]
    T = 1000*const.hbar**2/(2*const.m_eff_InAs*a**2*1e-18*const.e)   # Unit meV - input lattice constant in nm.
    
    mu, mu_sc, dis = 3.0, 3.0, 0.0 # meV
    delta = const.delta_InAs
    alpha = const.alpha_InAs/a
    Ez = 0
    
    sc_c = tls.SimpleNamespace(T=T, a=a, x_scale=0.0, y_scale=1.0, mu=mu, delta=delta, 
                           alpha=alpha, Ez=Ez)
    dW_ML = 50.0
    W_L = 100
    W_SC = 3500
    decay_length = nlc.decay_length(W_L + dW_ML, W_SC-W_L, sc_c)

    L = 1.5*decay_length

    dim_par = nlc.DimensionParameters(L=nlc.d_sys(L,a), W_L=nlc.d_sys(W_L,a), dW_ML = int(np.round(dW_ML/a)),
                                      dW_SC = int(np.round(10/a)), d=int(np.round(50/a)), dgT=int(np.round(30/a)),
                                      dL=int(np.round(0/a)), wgt=int(np.round(100/a)), dW_T=int(np.round(10/a)),
                                      W_SC=nlc.d_sys(W_SC, a))

    c = tls.SimpleNamespace(T=T, a=a, y_scale=1.0, x_scale=0.0, alpha=alpha, Ez=Ez, delta=delta,
                            mu=mu, mu_sc=mu_sc, dis=dis, disorder=nlc.disorder, salt='', tunnel_pots=nlc.tunnel_pots,
                            dim_par=dim_par)
    
    V_gtL = 0.0; V_gtR = 0.0
    gate_par = nlc.GateParameters(V_gtL=V_gtL, V_gtR=V_gtR)
    c.gate_par = gate_par
    
    syst = nlc.make_NSN_tunnel_gates(dim_par, plot_sys=False)
    plot_wf(syst, 0, 0, c, oversampling=10, method='cubic', colorbar=False, vmax=0.006, vmin=-0.006,
            cmap='RdBu_r', out_name=out_name);


# In[9]:

scatter_wf();


# ## No disorder, dispersion + nonloc conductance versus length and width

# In[10]:

def subplot_disp(ax, kvals, N_disp, sc_disp, xticks, yticks, ylim=(-1.25, 1.25), xlim=None, sharexax=True,
                 title=None, textloc=[0.5, 0.5], labelloc=[0.02, 0.7], label='$\mathrm{(a)}$',
                 xlabel=r'$ka$', ind_gap = None, flip=False, ylabel=None, shareyax=False):
    if ylabel is None:
        ylabel = r'$E/\Delta$'
    else:
        ax.set_ylabel(ylabel, fontsize=25)
    if N_disp is not None:
        pl = ax.plot(kvals, N_disp, c='grey', linestyle='dotted')
        if flip:
            pl = swap(pl)
        pl = ax.plot(kvals, -N_disp, c='grey', linestyle='dotted')
        if flip:
            pl = swap(pl)
    pl = ax.plot(kvals, sc_disp, 'k-')
    if flip:
        pl = swap(pl)
    # Plot a dotted line at E = delta and E = -delta, the bulk gap
    if not flip:
        ax.axhline(1.0, c="grey", linestyle='dashed')
        ax.axhline(-1.0, c="grey", linestyle='dashed')
    else:
        ax.axvline(1.0, c="grey", linestyle='dashed')
        ax.axvline(-1.0, c="grey", linestyle='dashed')
    if ind_gap is not None:
        if not flip:
            ax.axhline(ind_gap, c="grey", linestyle='dashdot')
            ax.axhline(-ind_gap, c="grey", linestyle='dashdot')
        else:
            ax.axvline(ind_gap, c="grey", linestyle='dashdot')
            ax.axvline(-ind_gap, c="grey", linestyle='dashdot')
    ax.set_ylim(ylim[0], ylim[1])
    if xlim is None:
        xlim=(min(kvals), max(kvals))
    ax.set_xlim(xlim[0], xlim[1])
    ax.tick_params(width=1.5, size=6, labelsize=23)
    set_ticks(ax, xticks, yticks)
    if sharexax:
        plt.setp(ax.get_xticklabels(), visible=False)
    else:
        ax.set_xlabel(xlabel, size=25)
    if shareyax:
        plt.setp(ax.get_yticklabels(), visible=False)
    else:
        ax.set_ylabel(ylabel, size=25)
    if title is not None:
        ax.text(textloc[0], textloc[1], title, transform=ax.transAxes, fontsize=20)
    ax.text(labelloc[0], labelloc[1], label, transform=ax.transAxes, fontsize=25)
            
                
def disp_nonloc_no_disorder():
    path = os.getcwd() + '/data/'

    # Data for short and intermediate junctions
    new_keys = [r'$\xi /2$', r'$\xi$', r'$2\xi$', r'$3\xi$']
    new_keys.reverse()
    out_name = 'loc_nonloc_lengths_W100nm_no_dis_mu3meV'
    out_name = out_name.replace('.',',')
    
    out_name_disp = 'all_widths_RSOI_no_Ez_mu3meV_a10nm'
    out_name_disp = out_name_disp.replace('.',',')
    name = 'disp_nonloc_no_dis.pdf'

    with open(path + out_name_disp + '.p', 'rb') as fp:
        data_disp = pickle.load(fp)
        W_Ns = pickle.load(fp) 
        keys_disp = pickle.load(fp)
    
    data, bias_vals, keys = read_for_waterfall(path+out_name)
    kvals1, sc_disp1, N_disp1 = data_disp[keys_disp[0]][0], data_disp[keys_disp[0]][1], data_disp[keys_disp[0]][2]
    data2, bias_vals2, new_keys = extract_cond_waterfall(data, bias_vals, keys, new_keys=new_keys,
                                                          plot='nonloc', trans='all')

    out_name = 'loc_nonloc_lengths_W200nm_no_dis_mu3meV'
    out_name = out_name.replace('.',',')
    data, bias_vals, keys = read_for_waterfall(path+out_name)
    kvals3, sc_disp3, N_disp3 = data_disp[keys_disp[2]][0], data_disp[keys_disp[2]][1], data_disp[keys_disp[2]][2]
    data4, bias_vals4, new_keys = extract_cond_waterfall(data, bias_vals, keys, new_keys=new_keys,
                                                          plot='nonloc', trans='all')
    # Long junction data
    out_name = 'W200nm_long_junction_mu42meV_delta2meV'
    out_name = out_name.replace('.',',')
    with open(path + out_name + '.p', 'rb') as fp:
        long_data = pickle.load(fp)
        long_bias_vals = pickle.load(fp) 
        long_keys = pickle.load(fp)
        long_dis_info = pickle.load(fp)
    dis_keys = list(long_data.keys())
    long_cond_data = long_data[dis_keys[0]][0]
    long_new_keys = [r'$\xi /2$', r'$\xi$', r'$2\xi$', r'$3\xi$']
    data6, bias_vals6, long_new_keys= extract_cond_waterfall(long_cond_data, long_bias_vals, long_keys,
                                                         new_keys=long_new_keys, plot='nonloc', trans='all')
    
    out_name_disp = 'long-junction_RSOI_no_Ez_Delta2meV_mu4,2meV_a20nm'
    with open(path + out_name_disp + '.p', 'rb') as fp:
        long_data_disp = pickle.load(fp)
        long_W_Ns = pickle.load(fp) 
        long_keys_disp = pickle.load(fp)
    kvals5, sc_disp5, N_disp5 = long_data_disp[long_keys_disp[0]][0], long_data_disp[long_keys_disp[0]][1],                                 long_data_disp[long_keys_disp[0]][2]
    
    
    # Read the induced gap from file
    # Short and intermediate
    out_name_gaps = 'induced_gap_meV_all_widths_RSOI_no_Ez_mu3meV_a10nm'
    with open(path + out_name_gaps + '.p', 'rb') as fp:
        data_gaps = pickle.load(fp)
        W_Ns_gaps = pickle.load(fp) 
        keys_gaps = pickle.load(fp)
    # Long
    out_name_gaps = 'induced_gap_meV_200nm_RSOI_no_Ez__Delta2meV_mu4,2meV_a20nm'
    with open(path + out_name_gaps + '.p', 'rb') as fp:
        long_data_gaps = pickle.load(fp)
        long_W_Ns_gaps = pickle.load(fp) 
        long_keys_gaps = pickle.load(fp)
    
    T=30; Elim=[-1.5, 1.5]; ylim1=[1.5, 4]
    Elim_long = [-1.2, 1.2]
    ylim2=None; ylim3=[1.5, 4]; ylim4=None; ylim5=[1.5, 4]; ylim6=None
    offset_step1=0.0;
    blims=[-0.3, 0.3]; blims_long = [-3.0, 3.0]
    legend_pos=(0.6, 1.0)
    plot = 'nonloc'; offset_step = 0.0
    klims = [0.0, 0.07]
    xticks_disp = [0, 0.03, 0.06]
    yticks_disp = [-1, 0, 1]
    yticks_disp_long = [-1, 0, 1]

    fig = plt.figure()
    fig.set_size_inches(7.5, 12)
    gs = gridspec.GridSpec(3, 2, height_ratios=[0.4, 0.4, 1], width_ratios=[0.66, 1])
    
    ax1 = plt.subplot(gs[0, 0])
    ind_gap = data_gaps[keys_gaps[0]]/0.2
    subplot_disp(ax1, kvals1/10, N_disp1, sc_disp1, xticks_disp, yticks_disp, label='$\mathrm{(a)}$',
                 ylim=Elim, xlim=klims, labelloc=[0.8, 0.68], title=r'$E_{\mathrm{Th}}/\Delta \approx 2$',
                 textloc=[0.05, 0.45], xlabel=r'$k \ [\mathrm{nm}^{-1}]$', ind_gap=ind_gap)
    ax1.axhspan(-ind_gap, -1, alpha=0.1, color=(1, 0.4, 0))
    ax1.axhspan(ind_gap, 1, alpha=0.1, color=(1, 0.4, 0))
    
    ax2 = plt.subplot(gs[0, 1])
    cond_waterfall(ax2, data2, bias_vals, new_keys, blims, flip=True, ylim=Elim, xlim=[-0.6, 1.8],
                   plot='nonloc', xticks=[-0.5, 0, 0.5, 1, 1.5], xlabel=r'$G_{21} \ [e^2/h]$',
                   offset_step=offset_step, T=T, title=' ', yticks=yticks_disp, ylabel=' ', shareyax=True, 
                   legend=True, textloc=[0.3, 0.3], ncol=4, sharexax=True, insert_title=True, 
                   yaxis_pos='left', label='$\mathrm{(b)}$', labelloc=[0.87, 0.68],
                   legend_pos=(-0.75, 1.25), ind_gap=ind_gap)
    ax2.axhspan(-ind_gap, -1, alpha=0.1, color=(1, 0.4, 0))
    ax2.axhspan(ind_gap, 1, alpha=0.1, color=(1, 0.4, 0))
    
    ax3 = plt.subplot(gs[1, 0])
    ind_gap = data_gaps[keys_gaps[2]]/0.2
    subplot_disp(ax3, kvals3/10, N_disp3, sc_disp3, xticks_disp, yticks_disp,
                 ylim=Elim, xlim=klims, label='$\mathrm{(c)}$', labelloc=[0.8, 0.68], 
                 title=r'$E_{\mathrm{Th}}/\Delta \approx 0.4$',
                 textloc=[0.05, 0.45], xlabel=r'$k \ [\mathrm{nm}^{-1}]$',
                 ind_gap=ind_gap)
    ax3.axhspan(-ind_gap, -1, alpha=0.1, color=(1, 0.4, 0))
    ax3.axhspan(ind_gap, 1, alpha=0.1, color=(1, 0.4, 0))

    ax4 = plt.subplot(gs[1, 1])
    ax4 = cond_waterfall(ax4, data4, bias_vals, new_keys, blims, flip=True, ylim=Elim, xlim=[-0.6, 1.8],
                         plot='nonloc', xticks=[-0.5, 0, 0.5, 1, 1.5], xlabel=r'$G_{21} \ [e^2/h]$',
                         offset_step=offset_step, T=T, title=' ', yticks=yticks_disp, ylabel=' ', shareyax=True,
                        legend=False, textloc=[0.3, 0.8], insert_title=True, sharexax=True,
                         yaxis_pos='left', label='$\mathrm{(d)}$', labelloc=[0.85, 0.68],
                        ind_gap=ind_gap)
    ax4.axhspan(-ind_gap, -1, alpha=0.1, color=(1, 0.4, 0))
    ax4.axhspan(ind_gap, 1, alpha=0.1, color=(1, 0.4, 0))
    
    ax5 = plt.subplot(gs[2, 0])
    ind_gap = long_data_gaps[long_keys_gaps[0]]/2
    subplot_disp(ax5, kvals5/20, N_disp5, sc_disp5, xticks_disp, yticks_disp_long, sharexax=False,
                 ylim=Elim_long, xlim=klims, label='$\mathrm{(e)}$', labelloc=[0.1, 0.85],
                 title=r'$E_{\mathrm{Th}}/\Delta \approx 0.04$',
                 textloc=[0.05, 0.64], xlabel=r'$k \ [\mathrm{nm}^{-1}]$',
                 ind_gap=ind_gap)
    ax5.axhspan(-ind_gap, -1, alpha=0.1, color=(1, 0.4, 0))
    ax5.axhspan(ind_gap, 1, alpha=0.1, color=(1, 0.4, 0))
    
    ax6 = plt.subplot(gs[2, 1])
    T = 100
    ax6 = cond_waterfall(ax6, data6, bias_vals6, new_keys, blims_long, flip=True, ylim=Elim_long, xlim=[-0.6, 1.8],
                         plot='nonloc', xticks=[-0.5, 0, 0.5, 1, 1.5], xlabel=r'$G_{21} \ [e^2/h]$', delta=2.0,
                         offset_step=offset_step, T=T, title=' ', yticks=yticks_disp, ylabel=' ', shareyax=True,
                        legend=False, textloc=[0.3, 0.8], insert_title=True,
                         yaxis_pos='left', label='$\mathrm{(f)}$', labelloc=[0.85, 0.58],
                        ind_gap=ind_gap)
    ax6.axhspan(-ind_gap, -1, alpha=0.1, color=(1, 0.4, 0))
    ax6.axhspan(ind_gap, 1, alpha=0.1, color=(1, 0.4, 0))

    plt.tight_layout()
    plt.subplots_adjust(wspace=0.05, hspace=0.05)
    plt.savefig(name, bbox_inches='tight')
    plt.show()


# In[11]:

disp_nonloc_no_disorder()


# ## Nonlocal conductance as a function of length at zero bias

# In[12]:

def suppression_nonloc_zero_bias():
    path = os.getcwd() + '/data/'
    plot_name = 'nonloc_vs_length_zero_bias.pdf'
    # Data for short and intermediate junctions
    new_keys = [r'$\xi /2$', r'$\xi$', r'$2\xi$', r'$3\xi$']
    new_keys.reverse()
    out_name = 'loc_nonloc_lengths_W100nm_no_dis_mu3meV'
    out_name = out_name.replace('.',',')


    data, bias_vals, keys = read_for_waterfall(path+out_name)
    data2, bias_vals2, new_keys2 = extract_cond_waterfall(data, bias_vals, keys, new_keys=new_keys,
                                                          plot='nonloc', trans='all')


    out_name = 'loc_nonloc_lengths_W200nm_no_dis_mu3meV'
    out_name = out_name.replace('.',',')
    data, bias_vals, keys = read_for_waterfall(path+out_name)
    data4, bias_vals4, new_keys4 = extract_cond_waterfall(data, bias_vals, keys, new_keys=new_keys,
                                                          plot='nonloc', trans='all')

    # Long junction data
    out_name = 'W200nm_long_junction_mu42meV_delta2meV'
    out_name = out_name.replace('.',',')
    with open(path + out_name + '.p', 'rb') as fp:
        long_data = pickle.load(fp)
        long_bias_vals = pickle.load(fp) 
        long_keys = pickle.load(fp)
        long_dis_info = pickle.load(fp)
    dis_keys = list(long_data.keys())
    long_cond_data = long_data[dis_keys[0]][0]
    long_new_keys = [r'$\xi /2$', r'$\xi$', r'$2\xi$', r'$3\xi$']
    data6, bias_vals6, long_new_keys= extract_cond_waterfall(long_cond_data, long_bias_vals, long_keys,
                                                         new_keys=long_new_keys, plot='nonloc', trans='all')

    T = 30
    delta = 2.0
    blims = [-delta-0.05, delta+0.05]
    # If no temperature specified, assume 0 temperature.
    if T is None:
        zero_index2 = int(len(bias_vals2)/2)
        zero_bias_data2 = [data2[key][zero_index2] for key in new_keys2]
        zero_index4 = int(len(bias_vals4)/2)
        zero_bias_data4 = [data4[key][zero_index4] for key in new_keys4]
        zero_index6 = int(len(bias_vals6)/2)
        zero_bias_data6 = [data6[key][zero_index6] for key in reversed(long_new_keys)]
    else:
        zero_bias_data2 = []
        for j, key in enumerate(new_keys2):
            temp_cond, temp_bias = temperature_cond_bias(bias_vals2, np.array(data2[key]), blims, T)
            zero_index2 = int(len(temp_bias)/2)
            zero_bias_point2 = temp_cond[zero_index2]
            zero_bias_data2.append(zero_bias_point2)
        zero_bias_data4 = []
        for j, key in enumerate(new_keys4):
            temp_cond, temp_bias = temperature_cond_bias(bias_vals4, np.array(data4[key]), blims, T)
            zero_index4 = int(len(temp_bias)/2)
            zero_bias_point4 = temp_cond[zero_index4]
            zero_bias_data4.append(zero_bias_point4)
        zero_bias_data6 = []
        for j, key in enumerate(reversed(long_new_keys)):
            temp_cond, temp_bias = temperature_cond_bias(bias_vals6, np.array(data6[key]), blims, T)
            zero_index6 = int(len(temp_bias)/2)
            zero_bias_point6 = temp_cond[zero_index6]
            zero_bias_data6.append(zero_bias_point6)

    zero_bias_data2 = zero_bias_data2[::-1]
    zero_bias_data4 = zero_bias_data4[::-1]
    zero_bias_data6 = zero_bias_data6[::-1]
    zero_bias_data = [zero_bias_data2, zero_bias_data4, zero_bias_data6]
    xi2 = 150
    x_vals2 = [0.5*xi2, 1*xi2, 2*xi2, 3*xi2]
    xi4 = 1443
    x_vals4 = [0.5*xi4, xi4, 2*xi4, 3*xi4]
    xi6 = 2310
    x_vals6 = [0.5*xi6, xi6, 2*xi6, 3*xi6]
    x_vals = [x_vals2, x_vals4, x_vals6]

    labels = [r'$E_{\mathrm{Th}}/\Delta \approx 2$', r'$E_{\mathrm{Th}}/\Delta \approx 0.4$',
             r'$E_{\mathrm{Th}}/\Delta \approx 0.04$']
    line_styles = ['ko', 'b^', 'rs']

    fig = plt.figure()
    fig.set_size_inches((7.5, 4))
    ax = fig.add_subplot(1, 1, 1)
    for i in range(3):
        label = labels[i]
        ax.plot(x_vals[i], zero_bias_data[i], line_styles[i], lw=2, label=label, markersize=10)
    ax.set_yscale('log')
    #ax.set_xlim((0, ))
    ax.tick_params(width=1.5, size=6, labelsize=23)
    set_ticks(ax, [0, 2000, 4000, 6000], [0, -1, -2, -3], logy=True)
    ax.set_ylabel(r'$G_{21} \ [e^2/h]$', size=25)
    ax.set_xlabel(r'$L \ [\mathrm{nm}]$', size=25)
    ax.legend(loc='upper right', fontsize=20)
    plt.savefig(plot_name, bbox_inches='tight')
    plt.show()


# In[13]:

suppression_nonloc_zero_bias()


# ## All widths, different disorder at a fixed length ($L \gg \xi$ )

# In[14]:

def loc_nonloc_with_dis_fixed_length():
    path = os.getcwd() + '/data/'
    out_name = 'All_Ws_MFPs_large_L_10configs_RSOI_a10nm'
    name = 'loc_nonloc_all_widths_fixed_length.pdf'

    new_keys = [r'$L$', r'$0.5L$', r'$0.3L$', r'$0.2L$', r'$0.1L$', r'$0.05L$']
    new_keys.reverse()

    with open(path + out_name + '.p', 'rb') as fp:
        data = pickle.load(fp)
        bias_vals = pickle.load(fp)
        keys = pickle.load(fp)
        dis_dict = pickle.load(fp)
        
    conf = 9
    
    def remove_entries(d):
        # Remove two curves, only show 4 MFPs per plot.
        del d[r'$0.3L$']
        del d[r'$0.05L$']
        return d
        
    diss, dis_keys = dis_dict[keys[0]]
    
    data3, bias_vals3, new_keys = extract_cond_waterfall(data[keys[0]][conf], bias_vals, dis_keys, 
                                                         new_keys=new_keys, plot='loc', trans='all')
    remove_entries(data3)
    data1, bias_vals1, new_keys = extract_cond_waterfall(data[keys[0]][conf], bias_vals, dis_keys, 
                                                         new_keys=new_keys, plot='nonloc', trans='all')
    remove_entries(data1)
    
    diss, dis_keys = dis_dict[keys[2]]
    data4, bias_vals4, new_keys = extract_cond_waterfall(data[keys[2]][conf], bias_vals, dis_keys,
                                                         new_keys=new_keys, plot='loc', trans='all')
    remove_entries(data4)
    data2, bias_vals2, new_keys = extract_cond_waterfall(data[keys[2]][conf], bias_vals, dis_keys,
                                                         new_keys=new_keys, plot='nonloc', trans='all')
    remove_entries(data2)
    new_keys = [r'$L$', r'$0.5L$', r'$0.2L$', r'$0.1L$']
    new_keys.reverse()
    
        
    T=30; xlim=[-1.25, 1.25]
    ylim1=[1.8, 4.2]; ylim2=[-0.15, 0.22]
    ylim5=[0.8, 4.2]; ylim6=[-0.3, 0.7]
    offset_step=0.0; blims=[-0.25, 0.25]
    legend_pos=(0.6, 1.0); legend = False
    fontsize=20; delta = 0.2
    yticks_left1 = [-0.1, 0.0, 0.1, 0.2]
    yticks_left2 = [2, 3, 4]
    yticks_right1 = [-0.2, 0, 0.2, 0.4, 0.6]
    yticks_right2 = [1, 2, 3, 4]
    xticks = [-1, 0, 1]
    
    fig = plt.figure()
    fig.set_size_inches(7.5, 5)
    ax1 = fig.add_subplot(2, 2, 1)
    ax1 = cond_waterfall(ax1, data1, bias_vals, new_keys, blims, xlim=xlim, ylim=ylim2, plot='nonloc',
                         offset_step=offset_step, T=T, title=r'$E_{\mathrm{Th}}/\Delta \approx 2$',
                         yticks=yticks_left1, legend=True, xticks=xticks,
                         legend_pos=(-0.4, 1.48), ncol=4, key_add='$l_{\mathrm{e}} = $ ', textloc=[0.2, 1.05], 
                         sharexax=True, insert_title=True, yaxis_pos='left', right_yaxis=False, 
                         labelloc=[0.2, 0.1], label='$\mathrm{(a)}$', ylabel=r'$G_{21} \ [e^2/h]$',
                         ind_gap=None)
    ax2 = fig.add_subplot(2, 2, 2)
    ax2 = cond_waterfall(ax2, data2, bias_vals, new_keys, blims, xlim=xlim, ylim=ylim6, plot='nonloc',
                         offset_step=offset_step, T=T, title=r'$E_{\mathrm{Th}}/\Delta \approx 0.4$',
                         yticks=yticks_right1, legend=False, xticks=xticks,
                         textloc=[0.2, 1.05], sharexax=True, insert_title=True, yaxis_pos='left', 
                         right_yaxis=True, label='$\mathrm{(b)}$', ylabel=r' ',
                         ind_gap=None, labelloc=[0.30, 0.1])
    ax3 = fig.add_subplot(2, 2, 3)
    ax3 = cond_waterfall(ax3, data3, bias_vals, new_keys, blims, xlim=xlim, ylim=ylim1, plot='loc',
                         offset_step=offset_step, T=T, title=' ', xticks=xticks,
                         yticks=yticks_left2, legend=False, 
                         textloc=[0.32, 0.2], sharexax=False, insert_title=False, yaxis_pos='left', 
                         right_yaxis=False, label='$\mathrm{(c)}$', ylabel=r'$G_{11} \ [e^2/h]$',
                         ind_gap=None, labelloc=[0.2, 0.1])
    ax4 = fig.add_subplot(2, 2, 4)
    ax4 = cond_waterfall(ax4, data4, bias_vals, new_keys, blims, xlim=xlim, ylim=ylim5, plot='loc',
                         offset_step=offset_step, T=T, title=' ', yticks=yticks_right2, legend=False,
                         xticks=xticks,
                         textloc=[0.32, 0.15], sharexax=False, insert_title=False, yaxis_pos='left', 
                         right_yaxis=True, label='$\mathrm{(d)}$', ylabel=r' ',
                         ind_gap=None, labelloc=[0.33, 0.1])
    
    plt.tight_layout()
    plt.subplots_adjust(wspace=0.25, hspace=0.07)
    plt.savefig(name, bbox_inches='tight')
    plt.show()


# In[15]:

loc_nonloc_with_dis_fixed_length()


# ##  Local and nonlocal color plots, varying width, no disorder 

# In[16]:

def subplot_color(fig, ax, cbar_ticks, key, data, bias_vals, Ez_dict, E_T_dict, conf, sharexax=True,
                  xlabel=False, ylabel=False, barlabel=r"$G \ [e^2/h]$", T=None, c_lims=None, plot_ind=0,
                 labelloc=[0.01, 0.8], label='$\mathrm{(a)}$', title=None, titleloc=[0.2, 0.8],
                 width_label=None, widthloc=[0.1, 0.8], width_color='w', include_xticks=True):
    """plot_ind = 0 for nonloc, = 3 for loc."""
    Ez_range = Ez_dict[key]
    E_T = E_T_dict[key]
    blims=None
    delta = 0.2
    
    if conf: # Plot a particular disorder configuration
        dat = np.reshape(np.array(data[key]['conf'+str(conf)]), (len(Ez_dict[key]), len(bias_vals), 6))
    else:  # If no config specified, average over them
        dat = sum([np.reshape(np.array(data[key][conf]), (len(Ez_dict[key]), len(bias_vals), 6))
                  for conf in data[key].keys()])/len(data[key].keys())
    if T is None: # Assume zero temperature
        cond = dat[:, :, plot_ind]
        energies = bias_vals
    else: # Nonzero temperature given
        if blims is None:
            blims = (-delta-0.015, delta+0.015)
        cond_raw = dat[:, :, plot_ind]
        cond = np.vstack([temperature_cond_bias(bias_vals, row, blims, T)[0] for row in cond_raw])
        energies = temperature_cond_bias(bias_vals, cond_raw[0, :], blims, T)[1]
    
    if include_xticks:
        xticks = [0, 2, 4, 6, 8, 10]
    else:
        xticks = [0, 4, 8]
        
    yticks = [-1, 0, 1]

    if c_lims is not None:
        normalizer = MidpointNormalize(midpoint=0, vmin=c_lims[0], vmax=c_lims[1])
    else:
        normalizer = MidpointNormalize(midpoint=0)

    plt.imshow(cond.T[::-1, :], extent=[min(Ez_range/E_T), max(Ez_range/E_T), min(energies/delta),
                                 max(energies/delta)], cmap='RdBu_r', aspect='auto',
               norm=normalizer)
    set_ticks(ax, xticks, yticks)
    ax.set_xlim(min(Ez_range/E_T), max(Ez_range/E_T))
    ax.tick_params(width=1.5, size=6, labelsize=20)
    ax.set_ylim(min(energies/delta), max(energies/delta))

    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.1)
    cb = fig.colorbar(ax.images[0], cax=cax, ticks=cbar_ticks)
    cb.set_ticklabels([latex(i) for i in cbar_ticks])
    if barlabel:
        cb.set_label(barlabel, fontsize=25)
    cax.tick_params(width=1.5, size=6, labelsize=20)
    if sharexax:
        plt.setp(ax.get_xticklabels(), visible=False)
    else:
        if xlabel:
            ax.set_xlabel('$E_{\mathrm{Z}}/\delta$', fontsize=25)
    if ylabel:
        ax.set_ylabel('$E/\Delta$', fontsize=25)
    else: 
        plt.setp(ax.get_yticklabels(), visible=False)
    ax.text(labelloc[0], labelloc[1], label, transform=ax.transAxes, fontsize=25)
    if title:
        ax.text(titleloc[0], titleloc[1], title, transform=ax.transAxes, fontsize=25)
    if width_label:
        ax.text(widthloc[0], widthloc[1], width_label, transform=ax.transAxes, fontsize=20, color=width_color)
    
        
        
def get_line_data(data, key, conf, Ez_dict, bias_vals, slice_ind, T=None, blims=None):
    delta = 0.2
    if conf: # Plot a particular disorder configuration
        dat = np.reshape(np.array(data[key]['conf'+str(conf)]), (len(Ez_dict[key]), len(bias_vals), 6))
    else: 
        dat = sum([np.reshape(np.array(data[key][conf]), (len(Ez_dict[key]), len(bias_vals), 6))
                      for conf in data[key].keys()])/len(data[key].keys())
    if T is None: # Assume zero temperature
        nonloc_cond = dat[:, :, 0]
        energies = bias_vals
    else: # Nonzero temperature given
        if blims is None:
            blims = (-delta-0.015, delta+0.015)
        nonloc_cond_raw = dat[:, :, 0]
        nonloc_cond = np.vstack([temperature_cond_bias(bias_vals, row, blims, T)[0] for row in nonloc_cond_raw])
        energies = temperature_cond_bias(bias_vals, nonloc_cond_raw[0, :], blims, T)[1]
    
    nonloc_line = nonloc_cond[slice_ind, :]
    return nonloc_line, energies

def loc_nonloc_no_disorder_color():
    
    path = os.getcwd() + '/data/'
    name = 'loc_nonloc_no_disorder_color.pdf'
    out_name = 'Open_all_widths_longer_systems_no_disorder_RSOI_mu3meV_a10nm'
    # Lengths are 10 xi, 6 xi and 3 xi
    out_name = out_name.replace('.',',')
    # Read data
    with open(path + out_name + '.p', 'rb') as fp:
        data = pickle.load(fp)
        Ez_dict = pickle.load(fp)
        E_T_dict = pickle.load(fp)
        bias_vals = pickle.load(fp)
        keys = pickle.load(fp)
        dis_dict = pickle.load(fp)
        gate_par = pickle.load(fp)
        L_vals = pickle.load(fp)
    
    R_plot_ind=0; L_plot_ind=3
    delta=0.2
    conf=0
    T = 30
    yticks_disp = [-1, 0,  1]
    cbar_ticks1 = [-0.2, 0, 0.2, 0.4]
    L_cbar_ticks = [0,  2,  4]
    L_c_lims = [0, 4]
    R_c_lims = [-0.2, 0.2]
    
    fig = plt.figure()
    fig.set_size_inches(7.0, 9)
    gs = gridspec.GridSpec(4, 21, height_ratios=[1, 1, 0.075, 0.7])

    ax2 = plt.subplot(gs[0, :])
    subplot_color(fig, ax2, cbar_ticks1, keys[0], data, bias_vals, Ez_dict, E_T_dict, conf, sharexax=False,
                  c_lims=R_c_lims, T=T, plot_ind=R_plot_ind, labelloc=[0.02, 0.5], label='$\mathrm{(a)}$',
                  width_label=r'$W = 100 \ \mathrm{nm}$', widthloc=[0.32, 0.8], ylabel=True, 
                  barlabel=r"$G_{21} \ [e^2/h]$")

    ax4 = plt.subplot(gs[1, :])
    subplot_color(fig, ax4, cbar_ticks1, keys[2], data, bias_vals, Ez_dict, E_T_dict, conf, sharexax=False, 
                  xlabel=True, c_lims=R_c_lims, T=T, plot_ind=R_plot_ind, labelloc=[0.02, 0.5], 
                  label='$\mathrm{(b)}$', width_label=r'$W = 200 \ \mathrm{nm}$', widthloc=[0.32, 0.8],
                 ylabel=True, barlabel=r"$G_{21} \ [e^2/h]$")
    
    # Line cuts to show the rectifier
    ax5 = plt.subplot(gs[3, 2:21])
    slice_ind = 57
    cond, energies = get_line_data(data, keys[0], conf, Ez_dict, bias_vals, slice_ind, T)
    ax5.plot(energies/delta, cond, label=r'$W=100 \ \mathrm{nm}$', color='blue')
    slice_ind = 53
    cond, energies = get_line_data(data, keys[2], conf, Ez_dict, bias_vals, slice_ind, T)
    ax5.plot(energies/delta, cond, label=r'$W=200 \ \mathrm{nm}$', color='red')
    ax5.axhline(0, c="black", linestyle='dotted')
    set_ticks(ax5, [-0.03, 0, 0.03], [-0.04,  0, 0.04])
    ax5.set_xlim(-0.04, 0.04)
    ax5.set_ylim(-0.04, 0.04)
    ax5.legend(loc=(0.52, 0.02), fontsize=16)
    ax5.tick_params(width=1.5, size=6, labelsize=23)
    ax5.set_xlabel(r'$E/\Delta$', fontsize=25)
    ax5.set_ylabel(r'$G_{21} \ [e^2/h]$', fontsize=25)
    ax5.text(0.02, 0.7, '$\mathrm{(c)}$', transform=ax5.transAxes, fontsize=25)
    
    plt.tight_layout()
    plt.subplots_adjust(wspace=0.11, hspace=0.25)
    plt.savefig(name, bbox_inches='tight')
    plt.show()


# In[17]:

loc_nonloc_no_disorder_color()


# ##  Nonlocal conductance color plot in quasi-ballistic and diffusive limit

# In[18]:

def nonloc_disorder_color():
    
    path = os.getcwd() + '/data/'
    out_name = 'Open_all_widths_MFPs0.2L_10configs'
    out_name = out_name.replace('.',',')
    name = 'nonloc_qb_and_diff_color.pdf'

    # Read data
    with open(path + out_name + '.p', 'rb') as fp:
        data_qb = pickle.load(fp)
        Ez_dict_qb = pickle.load(fp)
        E_T_dict_qb = pickle.load(fp)
        bias_vals_qb = pickle.load(fp)
        keys_qb = pickle.load(fp)
        dis_dict_qb = pickle.load(fp)
        
    path = os.getcwd() + '/data/'
    out_name = 'Open_all_widths_MFPs0.2W_10configs_single_mode_shorter_lengths_mu16meV'
    out_name = out_name.replace('.',',')
    
    # Read data
    with open(path + out_name + '.p', 'rb') as fp:
        data_d = pickle.load(fp)
        Ez_dict_d = pickle.load(fp)
        E_T_dict_d = pickle.load(fp)
        bias_vals_d = pickle.load(fp)
        keys_d = pickle.load(fp)
        dis_dict_d = pickle.load(fp)
        gate_par_d = pickle.load(fp)
        L_vals_d = pickle.load(fp)
        
    conf_qb=3; conf_d=7; delta=0.2; T=30
    cbar_ticks1 = [-0.2, 0, 0.2, 0.4]
    cbar_ticks2 = [-0.1, 0, 0.1]
    cbar_ticks3 = [-0.2, 0, 0.2, 0.4]
    cbar_ticks4 = [-0.1, 0, 0.1]
    climsL = [-0.2, 0.2]
    climsR = [-0.1, 0.1]
    
    fig = plt.figure()
    fig.set_size_inches(7.5, 6)
    
    ax1 = fig.add_subplot(2, 2, 1)
    subplot_color(fig, ax1, cbar_ticks1, keys_qb[0], data_qb, bias_vals_qb, Ez_dict_qb, E_T_dict_qb, conf_qb, 
                  sharexax=False, ylabel=True, barlabel=False, T=T, c_lims=climsL,
                 labelloc=[0.02, 0.45], label='$\mathrm{(a)}$', title=r'$\mathrm{quasiballistic}$',
                  titleloc=[0.2, 1.25], width_label=r'$W = 100 \ \mathrm{nm}$', widthloc=[0.15, 1.03],
                 width_color='k')
    
    ax2 = fig.add_subplot(2, 2, 2)
    subplot_color(fig, ax2, cbar_ticks2, keys_d[0], data_d, bias_vals_d, Ez_dict_d, E_T_dict_d, conf_d,
                 sharexax=False, T=T, c_lims=climsR, labelloc=[0.02, 0.45], label='$\mathrm{(b)}$', 
                  title=r'$\mathrm{diffusive}$', titleloc=[0.3, 1.25], width_label=r'$W = 100 \ \mathrm{nm}$', 
                  widthloc=[0.15, 1.03], width_color='k', barlabel=r"$G_{21} \ [e^2/h]$")
    
    ax3 = fig.add_subplot(2, 2, 3)
    subplot_color(fig, ax3, cbar_ticks3, keys_qb[2], data_qb, bias_vals_qb, Ez_dict_qb, E_T_dict_qb, conf_qb,
                  sharexax=False, xlabel=True, ylabel=True, barlabel=False, T=T, c_lims=climsL,
                 labelloc=[0.02, 0.45], label='$\mathrm{(c)}$', width_label=r'$W = 200 \ \mathrm{nm}$', 
                  widthloc=[0.15, 1.03], width_color='k', include_xticks=False)
    
    ax4 = fig.add_subplot(2, 2, 4)
    subplot_color(fig, ax4, cbar_ticks4, keys_d[2], data_d, bias_vals_d, Ez_dict_d, E_T_dict_d, conf_d, 
                  sharexax=False, xlabel=True, T=T, c_lims=climsR, labelloc=[0.02, 0.45], label='$\mathrm{(d)}$',
                  width_label=r'$W = 200 \ \mathrm{nm}$', widthloc=[0.15, 1.03], width_color='k', include_xticks=False,
                  barlabel=r"$G_{21} \ [e^2/h]$")
                
    plt.tight_layout()
    plt.subplots_adjust(hspace=0.35, wspace=0.3)
    plt.savefig(name, bbox_inches='tight')
    plt.show()


# In[19]:

nonloc_disorder_color()


# ## Local and nonlocal conductance in a 'broken wire'

# In[20]:

def get_color_data(data, data_tunnel, bias_vals, bias_vals_tunnel, key, T, c_lims, c_lims_loc, blims=None):
    delta = 0.2
    if c_lims is not None:
        normalizer = MidpointNormalize(midpoint=0, vmin=c_lims[0], vmax=c_lims[1])
    else:
        normalizer = MidpointNormalize(midpoint=0)
    if c_lims_loc is not None:
        loc_normalizer = MidpointNormalize(vmin=c_lims_loc[0], vmax=c_lims_loc[1])
    if T is None: # Assume zero temperature
        nonloc = np.array(data[key])[:, :, 0]
        loc = np.array(data_tunnel[key])[:, :, 3]
        energies = bias_vals
        energies_tunnel = bias_vals
    else: # Nonzero temperature given
        if blims is None:
            blims = (-delta-0.015, delta+0.015)
        nonloc_raw = np.array(data[key])[:, :, 0]
        nonloc = np.vstack([temperature_cond_bias(bias_vals, row, blims, T)[0] for row in nonloc_raw])
        loc_raw = np.array(data_tunnel[key])[:, :, 3]
        loc = np.vstack([temperature_cond_bias(bias_vals_tunnel, row, blims, T)[0] for row in loc_raw])
        energies = temperature_cond_bias(bias_vals, nonloc_raw[0, :], blims, T)[1]
        energies_tunnel = temperature_cond_bias(bias_vals_tunnel, loc_raw[0, :], blims, T)[1]
        
    return nonloc, loc, energies_tunnel, energies, normalizer, loc_normalizer


def spectrum_loc_nonloc_broken_wire():
    path = os.getcwd() + '/data/broken_wire/'
    plot_name = 'broken_wire.pdf'
    T = 30; delta = 0.2

    name = 'invariant_broken_wire'
    with open(path + name + '.p', 'rb') as fp:
        data_determinant = pickle.load(fp)
        Ez_fine_dict = pickle.load(fp)

    name = 'spectrum_broken_wire_data'
    with open(path + name + '.p', 'rb') as fp:
        data_dict = pickle.load(fp)
        Ez_dict_spec = pickle.load(fp)
        E_T = pickle.load(fp)
        pot_keys = pickle.load(fp)

    name = 'cond_data_open'
    with open(path + name + '.p', 'rb') as fp:
        data_cond = pickle.load(fp)
        Ez_dict = pickle.load(fp)
        E_T = pickle.load(fp)
        bias_vals = pickle.load(fp)
        pot_keys = pickle.load(fp)

    name = 'cond_data_tunnel'
    with open(path + name + '.p', 'rb') as fp:
        data_cond_tunnel = pickle.load(fp)
        Ez_dict_tunnel = pickle.load(fp)
        E_T = pickle.load(fp)
        bias_vals_t = pickle.load(fp)
        pot_keys = pickle.load(fp)

    spectrum_data0 = data_dict[pot_keys[0]]
    spectrum_data2 = data_dict[pot_keys[1]]
    Ez_range = Ez_dict[pot_keys[0]]
    Ez_range_spec = Ez_dict_spec[pot_keys[0]]
    Ez_range_tunnel = Ez_dict_tunnel[pot_keys[0]]
    c_lims = [-0.2, 0.2]
    c_lims_loc = [0, 0.5]
    delta = 0.2
    blims = None

    nonloc0, loc0, energies0_tunnel, energies0, normalizer, loc_normalizer = get_color_data(data_cond,
                                                                                data_cond_tunnel, bias_vals, 
                                                                                bias_vals_t, pot_keys[0], T,
                                                                                c_lims,
                                                                                c_lims_loc)
    nonloc2, loc2, energies2_tunnel, energies2, normalizer, loc_normalizer = get_color_data(data_cond,
                                                                                data_cond_tunnel, bias_vals, 
                                                                                bias_vals_t, pot_keys[1], T,
                                                                                c_lims, c_lims_loc)

    tqpt_point0 = Ez_fine_dict[pot_keys[0]][np.argmin(data_determinant[pot_keys[0]][0:round(len(
                        data_determinant[pot_keys[0]])/2)])]/E_T
    tqpt_point2 = Ez_fine_dict[pot_keys[1]][np.argmin(data_determinant[pot_keys[1]])]/E_T

    fig = plt.figure()
    fig.set_size_inches(7, 7.3)
    grid = gridspec.GridSpec(5, 4, height_ratios=[1, 0.05, 1, 0.05, 1], width_ratios=[40, 1, 40, 1],
                             wspace=0.05, hspace=0.05)


    yticks = [-1, 0,  1]
    xticks = [0, 2, 4, 6, 8]
    cbar_ticks_loc = [0, 0.2, 0.4, 0.6]
    cbar_ticks_nonloc = [-0.2, 0, 0.2, 0.4, 0.6]
    textloc = [0.03, 0.5]

    ax1 = fig.add_subplot(grid[0])
    spec_energies = np.sort(np.array([spec for spec in spectrum_data0]))
    ax1.plot(Ez_range_spec/E_T, spec_energies/delta, lw=2, color='k')
    ax1.set_ylim(-1., 1.)
    ax1.set_xlim(min(Ez_range/E_T), max(Ez_range/E_T))
    ax1.tick_params(width=1.5, size=6, labelsize=20)
    ax1.set_ylabel('$E/\Delta$', fontsize=20)
    set_ticks(ax1, xticks, yticks)
    plt.setp(ax1.get_xticklabels(), visible=False)
    ax1.axvspan(tqpt_point0, max(Ez_range/E_T), alpha=0.5, color=(1, 0.9, 0))
    ax1.axvspan(tqpt_point0, max(Ez_range/E_T), alpha=0.5, color=(1, 0.4, 0))
    ax1.text(textloc[0], textloc[1], '$\mathrm{(a)}$', transform=ax1.transAxes, fontsize=20)
    ax1.text(0.3, 1.1, r'$V_0 = 0 \ \mathrm{V}$', transform=ax1.transAxes, fontsize=23)
    ax1.axvline(tqpt_point0, c="black", linewidth=1.0)


    ax2 = fig.add_subplot(grid[8], sharex=ax1)
    im2 = ax2.imshow(loc0.T[::-1, :], extent=[min(Ez_range_tunnel/E_T), max(Ez_range_tunnel/E_T), 
                                              min(energies0/delta), 
                             max(energies0/delta)], cmap='RdBu_r', aspect='auto', norm=loc_normalizer)
    ax2.set_xlim(min(Ez_range_tunnel/E_T), max(Ez_range_tunnel/E_T))
    ax2.tick_params(width=1.5, size=6, labelsize=20)
    ax2.set_ylim(min(energies0/delta), max(energies0/delta))
    set_ticks(ax2, xticks, yticks)
    plt.setp(ax2.get_xticklabels(), visible=False)
    ax2.set_ylabel('$E/\Delta$', fontsize=20)
    ax2.text(textloc[0], textloc[1], '$\mathrm{(c)}$', transform=ax2.transAxes, fontsize=20)
    ax2.axvline(tqpt_point0, c="black", linewidth=1.0)

    ax3 = fig.add_subplot(grid[16], sharex=ax2)
    im3 = ax3.imshow(nonloc0.T[::-1, :], extent=[min(Ez_range/E_T), max(Ez_range/E_T), min(energies0/delta), 
                              max(energies0/delta)], cmap='RdBu_r', aspect='auto', norm=normalizer)
    ax3.set_xlim(min(Ez_range/E_T), max(Ez_range/E_T))
    ax3.tick_params(width=1.5, size=6, labelsize=20)
    ax3.set_xlabel('$E_{\mathrm{Z}}/\delta$', fontsize=20)
    ax3.set_ylabel('$E/\Delta$', fontsize=20)
    ax3.set_ylim(min(energies0/delta), max(energies0/delta))
    set_ticks(ax3, xticks, yticks)
    ax3.text(textloc[0], textloc[1], '$\mathrm{(e)}$', transform=ax3.transAxes, fontsize=20)
    ax3.axvline(tqpt_point0, c="black", linewidth=1.0)


    ax4 = fig.add_subplot(grid[2])
    spec_energies = np.sort(np.array([spec for spec in spectrum_data2]))
    ax4.plot(Ez_range_spec/E_T, spec_energies/delta, lw=2, color='k')
    ax4.set_ylim(-1., 1.)
    ax4.set_xlim(min(Ez_range/E_T), max(Ez_range/E_T))
    ax4.tick_params(width=1.5, size=6, labelsize=20)
    set_ticks(ax4, xticks, yticks)
    plt.setp(ax4.get_xticklabels(), visible=False)
    plt.setp(ax4.get_yticklabels(), visible=False)
    ax4.axvspan(2.2, max(Ez_range/E_T), alpha=0.5, color=(1, 0.9, 0))
    ax4.axvspan(tqpt_point2, max(Ez_range/E_T), alpha=0.5, color=(1, 0.4, 0))
    ax4.text(textloc[0], textloc[1], '$\mathrm{(b)}$', transform=ax4.transAxes, fontsize=20)
    ax4.text(0.15, 1.1, r'$V_0 = -4.5 \ \mathrm{mV}$', transform=ax4.transAxes, fontsize=23)
    ax4.axvline(2.2, c="black", linewidth=1.0)
    ax4.axvline(tqpt_point2, c="black", linewidth=1.0)


    ax5 = fig.add_subplot(grid[10], sharex=ax4)
    im5 = ax5.imshow(loc2.T[::-1, :], extent=[min(Ez_range_tunnel/E_T), max(Ez_range_tunnel/E_T), 
                                              min(energies2/delta), 
                             max(energies2/delta)], cmap='RdBu_r', aspect='auto', norm=loc_normalizer)
    ax5.set_xlim(min(Ez_range_tunnel/E_T), max(Ez_range_tunnel/E_T))
    ax5.tick_params(width=1.5, size=6, labelsize=20)
    ax5.set_ylim(min(energies2/delta), max(energies2/delta))
    set_ticks(ax5, xticks, yticks)
    plt.setp(ax5.get_xticklabels(), visible=False)
    plt.setp(ax5.get_yticklabels(), visible=False)
    ax5.text(textloc[0], textloc[1], '$\mathrm{(d)}$', transform=ax5.transAxes, fontsize=20)
    ax5.axvline(2.2, c="black", linewidth=1.0)
    ax5.axvline(tqpt_point2, c="black", linewidth=1.0)

    ColorAx3 = fig.add_subplot(grid[11])
    cbar3 = plt.colorbar(im5, cax=ColorAx3,ticks=cbar_ticks_loc)
    cbar3.set_ticklabels([latex(i) for i in cbar_ticks_loc])
    cbar3.set_label(r"$G_{11} \ [e^2/h]$", fontsize=20)
    ColorAx3.tick_params(width=1.5, size=6, labelsize=15)

    ax6 = fig.add_subplot(grid[18], sharex=ax5)
    im6 = ax6.imshow(nonloc2.T[::-1, :], extent=[min(Ez_range/E_T), max(Ez_range/E_T), min(energies2/delta), 
                              max(energies2/delta)], cmap='RdBu_r', aspect='auto', norm=normalizer)
    ax6.set_xlim(min(Ez_range/E_T), max(Ez_range/E_T))
    ax6.tick_params(width=1.5, size=6, labelsize=20)
    ax6.set_xlabel('$E_{\mathrm{Z}}/\delta$', fontsize=20)
    ax6.set_xlim(min(Ez_range/E_T), max(Ez_range/E_T))
    ax6.set_ylim(min(energies2/delta), max(energies2/delta))
    plt.setp(ax6.get_yticklabels(), visible=False)
    set_ticks(ax6, xticks, yticks)
    ax6.text(textloc[0], textloc[1], '$\mathrm{(f)}$', transform=ax6.transAxes, fontsize=20)
    ax6.axvline(2.2, c="black", linewidth=1.0)
    ax6.axvline(tqpt_point2, c="black", linewidth=1.0)

    ColorAx4= fig.add_subplot(grid[19])
    cbar4 = plt.colorbar(im6, cax=ColorAx4,ticks=cbar_ticks_nonloc)
    cbar4.set_ticklabels([latex(i) for i in cbar_ticks_nonloc])
    cbar4.set_label(r"$G_{21} \ [e^2/h]$", fontsize=20)
    ColorAx4.tick_params(width=1.5, size=6, labelsize=15)
    
    plt.subplots_adjust(hspace=0.3, wspace=0.3)
    plt.savefig(plot_name, bbox_inches='tight')
    plt.show()


# In[21]:

spectrum_loc_nonloc_broken_wire()


# ## Line cuts for an inhomogeneous system

# In[22]:

def loc_nonloc_line_cuts_inhomogeneous():
    path = os.getcwd() + '/data/broken_wire/'
    plot_name = 'inhomo_line_cuts.pdf'

    name = 'cond_data_open'
    with open(path + name + '.p', 'rb') as fp:
        data_cond = pickle.load(fp)
        Ez_dict = pickle.load(fp)
        E_T = pickle.load(fp)
        bias_vals = pickle.load(fp)
        pot_keys = pickle.load(fp)

    name = 'cond_data_tunnel'
    with open(path + name + '.p', 'rb') as fp:
        data_cond_tunnel = pickle.load(fp)
        Ez_dict_tunnel = pickle.load(fp)
        E_T = pickle.load(fp)
        bias_vals_t = pickle.load(fp)
        pot_keys = pickle.load(fp)
    
    T = 30; delta = 0.2; blims = None
    c_lims = [-0.2, 1]
    c_lims_loc = [0, 0.5]
    nonloc2, loc2, energies2_tunnel, energies2, normalizer, loc_normalizer = get_color_data(data_cond,
                                                                                data_cond_tunnel, bias_vals, 
                                                                                bias_vals_t, pot_keys[1], T,
                                                                                c_lims, c_lims_loc)

    yticks = [-1, -0.5, 0, 0.5, 1]
    xticks = [0, 1, 2, 3, 4, 5, 6, 7, 8]
    textloc = [0.03, 0.5]
    fontsize = 20
    Eticks = [-1, 0, 1]
    labelloc = (0.8, 0.8)
    Ez_range = Ez_dict[pot_keys[0]]
    
    
    fig = plt.figure()
    fig.set_size_inches(7, 4.3)
    
    ax1 = fig.add_subplot(1, 2, 1)
    pl = ax1.plot(energies2_tunnel/delta, loc2.T[:, 50], linewidth=1.5, c='black')
    pl = swap(pl)
    ax1.set_ylim(-1.05, 1.05)
    ax1.set_xlim(0, 0.4)
    ax1.set_ylabel(r'$E/\Delta$', size=fontsize)
    ax1.set_xlabel(r'$G_{11} \ [e^2/h]$', size=fontsize)
    plt.setp(ax1.get_xticklabels(), fontsize=fontsize)
    plt.setp(ax1.get_yticklabels(), fontsize=fontsize)
    set_ticks(ax1, [0, 0.1, 0.2, 0.3], Eticks)
    ax1.axhline(1.0, c="grey", linestyle='dashed')
    ax1.axhline(-1.0, c="grey", linestyle='dashed')
    ax1.text(labelloc[0], labelloc[1], r'$\mathrm{(a)}$', transform=ax1.transAxes, fontsize=25)
    ax1.tick_params(width=1.5, size=6, labelsize=20)
    ax1.text(0.34, 1.04, '$\mathrm{local}$',  transform=ax1.transAxes, fontsize=30)
    
    ax2 = fig.add_subplot(1, 2, 2, sharey=ax1)
    pl = ax2.plot(energies2/delta, nonloc2.T[:, 50], linewidth=1.5, c='black')
    pl = swap(pl)
    ax2.set_ylim(-1.05, 1.05)
    ax2.set_xlim(-0.1, 0.3)
    ax2.set_xlabel(r'$G_{21} \ [e^2/h]$', size=fontsize)
    plt.setp(ax2.get_xticklabels(), fontsize=fontsize)
    plt.setp(ax2.get_yticklabels(), visible=False)
    set_ticks(ax2, [-0.1, 0, 0.1, 0.2], Eticks)
    ax2.axhline(1.0, c="grey", linestyle='dashed')
    ax2.axhline(-1.0, c="grey", linestyle='dashed')
    ax2.axvline(0.0, c="black", linestyle='dashed')
    ax2.text(labelloc[0], labelloc[1], r'$\mathrm{(b)}$', transform=ax2.transAxes, fontsize=25)
    ax2.tick_params(width=1.5, size=6, labelsize=20)
    ax2.text(0.25, 1.04, '$\mathrm{nonlocal}$',  transform=ax2.transAxes, fontsize=30)
    
    plt.subplots_adjust(wspace=0.1)
    plt.savefig(plot_name, bbox_inches='tight')
    plt.show()


# In[23]:

loc_nonloc_line_cuts_inhomogeneous()


# ## Cooper pair splitter plots

# In[24]:

def color_cut(cut_ind, key, data, bias_vals, Ez_dict, E_T_dict, ind, delta = 0.2,
              blims=None, T=None, number_processes=10):
    Ez_range = Ez_dict[key]
    E_T = E_T_dict[key]
    
    # Data for all processes
    dat = np.reshape(np.array(data[key]), (len(Ez_dict[key]), len(bias_vals), number_processes))
    dat = np.concatenate((dat, np.reshape(dat[:, :, 0]+dat[:, :, 1], (len(Ez_dict[key]), len(bias_vals), 1))),
                         axis=2)
    line_cut = dat[cut_ind, :, ind]
    if T is None: # Assume zero temperature
        col = line_cut
        energies = bias_vals
    else: # Nonzero temperature given
        if blims is None:
            blims = (-delta-0.015, delta+0.015)
        col, energies = temperature_cond_bias(bias_vals, line_cut, blims, T)
    return col, energies, Ez_range[cut_ind], E_T
      

def clean_rectifier_line_cut(slice_ind):
    path = os.getcwd() + '/data/'
    out_name = 'Open_all_widths_longer_systems_no_disorder_RSOI_mu3meV_a10nm'
    # Lengths are 10 xi, 6 xi and 3 xi
    out_name = out_name.replace('.',',')
    # Read data
    with open(path + out_name + '.p', 'rb') as fp:
        data = pickle.load(fp)
        Ez_dict = pickle.load(fp)
        E_T_dict = pickle.load(fp)
        bias_vals = pickle.load(fp)
        keys = pickle.load(fp)
        dis_dict = pickle.load(fp)
    
    delta=0.2
    conf=0
    T = 30
    yticks_disp = [-1, 0, 1]
    
    cond, energies = get_line_data(data, keys[0], conf, Ez_dict, bias_vals, slice_ind, T)
    return cond, energies, Ez_dict[keys[0]][slice_ind], E_T_dict[keys[0]]


def disp_conds_cross_SOI():
    # Output name
    name = 'disp_conds_cross_SOI.pdf'
    # Get data
    path = os.getcwd() + '/data/'
    # Dispersions with RSOI
    out_name = 'W100nm_RSOI_and_Ezs_mu3meV_a10nm'
    with open(path + out_name + '.p', 'rb') as fp:
        RSOI_data = pickle.load(fp)
        RSOI_Ezs = pickle.load(fp)
        RSOI_keys = pickle.load(fp) 
        RSOI_disp_c = pickle.load(fp)
    out_name = 'Csplit_W200nm_mu0,5meV_a10nm_no_SOI'
    with open(path + out_name + '.p', 'rb') as fp:
        dope_disp_data = pickle.load(fp)
        dope_disp_keys = pickle.load(fp) 
        dope_disp_c = pickle.load(fp)
    path = os.getcwd() + '/../data/paper_data/'
    # Dispersions without RSOI
    out_name = 'Csplit_W200nm_WL200nm_mu0,8meV_musc0,5meV_no_SOI_L10xi'
    with open(path + out_name + '.p', 'rb') as fp:
        dope_cond_data = pickle.load(fp)
        dope_bias_vals = pickle.load(fp)
        dope_cond_keys = pickle.load(fp) 
        dope_dis_info = pickle.load(fp) 

    
    fig = plt.figure()
    fig.set_size_inches(7.5, 6.5)
    titles = [r'$E_{\mathrm{Z}} = 2.4 \delta$', r'$E_{\mathrm{Z}} = 2.2 \delta$']
    textlocs = ([0.37, 0.43], [0.33, 0.7])
    
    delta = 0.2
    a = 10 # lattice constant in nm
    col_title_left=None
    col_title_right=None
    titleloc_left = [0.16, 1.1]
    titleloc_right = [0.2, 1.1]
    # Dispersion RSOI
    i = 0
    key1 = RSOI_keys[1]
    ax1 = fig.add_subplot(2, 2, 1)
    ks1 = RSOI_data[key1][0]
    sc_disp1 = RSOI_data[key1][1]
    N_disp1 = RSOI_data[key1][2]
    ax1.axhspan(-0.81, -1, alpha=0.5, color=(1, 0.4, 0))
    ax1.axhspan(-0.2, -0.59, alpha=0.5, color=(1, 0.4, 0))
    subplot_disp(ax1, ks1/a, N_disp1, sc_disp1, [-0.04, 0, 0.04], [-1, 0, 1], ylim=(-1.05, 1.05),
                 xlim=(-0.07, 0.07),
                 sharexax=False, title=titles[i], textloc=textlocs[i], labelloc=[0.02, 0.7], label='$\mathrm{(a)}$',
                 ylabel=None, shareyax=False, xlabel=r'$k \ [\mathrm{nm}^{-1}]$')
    
    # Doping dispersion
    title = r'$E_{\mathrm{Th}}/\Delta \approx 0.5$'
    textloc = [0.08, 0.43]
    key2 = dope_disp_keys[0]
    ax2 = fig.add_subplot(2, 2, 3)
    ks2 = dope_disp_data[key2][0]
    sc_disp2 = dope_disp_data[key2][1]
    N_disp2 = dope_disp_data[key2][2]
    ax2.axhspan(-0.66, -1, alpha=0.5, color=(1, 0.4, 0))
    subplot_disp(ax2, ks2/a, N_disp2, sc_disp2, [-0.03, 0, 0.03], [-1, 0, 1], ylim=(-1.05, 1.05),
                 xlim=(-0.04, 0.04),
                 sharexax=False, title=title, textloc=textloc, labelloc=[0.02, 0.7], label='$\mathrm{(c)}$',
                 ylabel=None, shareyax=False, xlabel=r'$k \ [\mathrm{nm}^{-1}]$')
    
    ax3 = fig.add_subplot(2, 2, 2)
    cond, energies, Ez, level_spacing = clean_rectifier_line_cut(48)
    pl = ax3.plot(energies/delta, cond, 'k-')
    pl = swap(pl)
    ax3.axhspan(-0.81, -1, alpha=0.5, color=(1, 0.4, 0))
    ax3.axhspan(-0.2, -0.59, alpha=0.5, color=(1, 0.4, 0))
    ax3.set_ylim(-1.05, 1.05)
    ax3.set_xlim(-0.3, 1.1)
    ax3.axhline(1.0, c="grey", linestyle='dashed')
    ax3.axhline(-1.0, c="grey", linestyle='dashed')
    ax3.axvline(0, c="black", linestyle='dashed')
    ax3.tick_params(width=1.5, size=6, labelsize=23)
    set_ticks(ax3, [0, 0.5, 1], [-1, 0, 1])
    plt.setp(ax3.get_yticklabels(), visible=False)
    ax3.set_xlabel(r'$G_{21} \ [e^2/h]$', size=25)
    ax3.text(0.02, 0.7, '$\mathrm{(b)}$' , transform=ax3.transAxes, fontsize=25)
    
    
    # Nonlocal conductance
    T = 30
    blims = [-delta*1.5, delta*1.5]
    ax4 = fig.add_subplot(2, 2, 4)
    nonloc, tee, the, loc, ree, rhe = zip(*dope_cond_data[dope_dis_info[0][0]][0][dope_cond_keys[0]])
    nonloc = np.array(nonloc)
    if T is None: # Assume zero temperature
        nonloc_cond = nonloc
        energies = dope_bias_vals
    else: # Nonzero temperature given
        nonloc_cond, energies = temperature_cond_bias(dope_bias_vals, nonloc, blims, T)
    pl = ax4.plot(energies/delta, nonloc_cond, 'k-')
    pl = swap(pl)
    ax4.axhspan(-0.66, -1, alpha=0.5, color=(1, 0.4, 0))
    ax4.set_ylim(-1.05, 1.05)
    ax4.set_xlim(-0.3, 1.1)
    ax4.axhline(1.0, c="grey", linestyle='dashed')
    ax4.axhline(-1.0, c="grey", linestyle='dashed')
    ax4.axvline(0, c="black", linestyle='dashed')
    ax4.tick_params(width=1.5, size=6, labelsize=23)
    set_ticks(ax4, [0, 0.5, 1], [-1, 0, 1])
    plt.setp(ax4.get_yticklabels(), visible=False)
    ax4.set_xlabel(r'$G_{21} \ [e^2/h]$', size=25)
    ax4.text(0.02, 0.7, '$\mathrm{(d)}$' , transform=ax4.transAxes, fontsize=25)
    
    plt.tight_layout()
    plt.subplots_adjust(wspace=0.05, hspace=0.5)
    plt.savefig(name, bbox_inches='tight')
    plt.show()


# In[25]:

disp_conds_cross_SOI()


# ## Appendix figure: dispersions and DOS for short, intermediate and long junctions

# In[26]:

def subplot_disp(ax, disp, kvals, title=None, xlim=None, ylim=None, N_disp=None, include_yticks=True, ylabel=True,
                xticks=None, yticks=None, textloc=[0.69, 0.5], labelloc=[0.01, 0.45], label='$\mathrm{(a)}$',
                col_title=None, titleloc=[0.2, 1.1], xlabel=r'$ka$'):
    if ylabel:
        ax.set_ylabel(r'$E/\Delta$', fontsize=20)
    if xlabel:
        ax.set_xlabel(xlabel, fontsize=20)
    if N_disp is not None:
        ax.plot(kvals, N_disp, c='grey', linestyle='dotted')
        ax.plot(kvals, -N_disp, c='grey', linestyle='dotted')
    ax.plot(kvals, disp, 'k-')

    if title is not None:
        ax.text(textloc[0], textloc[1], title, transform=ax.transAxes,
                fontsize=20)
    # Plot a dotted line at E = delta and E = -delta, the bulk gap
    ax.axhline(1.0, c="grey", linestyle='dashed')
    ax.axhline(-1.0, c="grey", linestyle='dashed')
    ax.tick_params(width=1.5, size=6, labelsize=23)
    plt.setp(ax.get_xticklabels(), fontsize=20)
    plt.setp(ax.get_yticklabels(), fontsize=20)
    if yticks is None:
        plt.setp(ax.get_yticklabels(), visible=False)
    else:
        ax.set_yticks(yticks)
    if xticks is None:
        plt.setp(ax.get_xticklabels(), visible=False)
    else:
        ax.set_xticks(xticks)
    ax.set_ylim(ylim[0], ylim[1])
    ax.set_xlim(xlim[0], xlim[1])
    ax.text(labelloc[0], labelloc[1], label, transform=ax.transAxes, fontsize=20)
    set_ticks(ax, xticks, yticks)
    if col_title:
        ax.text(titleloc[0], titleloc[1], col_title, transform=ax.transAxes, fontsize=23)

def subplot_DOS(ax, DOS, energies, delta, title=None, Elim=[-0.21, 0.21], DOSlim=[0, 50], gammas=None,
               labelloc=[0.01, 0.45], label='$\mathrm{(a)}$'):
    ax.set_xlabel(r'$\rho$ [$1/2\pi \hbar$]', fontsize=20)
    xticks = [5, 15, 25, 35]
    yticks = []
    if gammas is None: # No broadening
        plot_energies = energies/delta
        pl = ax.plot(plot_energies, DOS, 'k-')
        pl = swap(pl)
        
    else: # broadening
        linestyles = ['k-', 'k--']
        for gamma, ls in zip(gammas, linestyles[:len(gammas)]):
            DElims = Elim
            conv_DOS, plot_energies = DOS_Lorentzian(energies, DOS, DElims, gamma)
            plot_energies = plot_energies/delta
            pl = ax.plot(plot_energies, conv_DOS, ls)
            pl = swap(pl)
    ax.set_title(title, fontsize=20)
    ax.tick_params(width=1.5, size=6, labelsize=20)
    # Plot a dotted line at E = delta and E = -delta, the bulk gap
    ax.axhline(delta/delta, c="grey", linestyle='dashed')
    ax.axhline(-delta/delta, c="grey", linestyle='dashed')
    ax.set_ylim(Elim[0]/delta, Elim[1]/delta)
    ax.set_xlim(DOSlim[0], DOSlim[1])
    plt.setp(ax.get_xticklabels(), fontsize=20)
    ax.set_yticks([]) # remove y-ticks (right column)
    ax.text(labelloc[0], labelloc[1], label, transform=ax.transAxes, fontsize=18)
    set_ticks(ax, xticks, yticks)

def disp_dos_plot():
    name = 'disp_dos_appendix.pdf'
    # Get data
    path = os.getcwd() + '/data/'
    # Dispersions
    out_name = 'short_int_long_dispersions'
    with open(path + out_name + '.p', 'rb') as fp:
        disp_data = pickle.load(fp)
        keys = pickle.load(fp) 
        
    # DOS
    out_name = 'short_int_long_DOS'
    with open(path + out_name + '.p', 'rb') as fp:
        dos_data = pickle.load(fp)
        keys = pickle.load(fp) 
        deltas = pickle.load(fp)
        
    
    fig = plt.figure()
    fig.set_size_inches(7.5, 6)
    gammas = ([deltas['100.0']/300],
              [deltas['250.0']/300, deltas['250.0']/30],
              [deltas['800.0']/300])
    
    disp_xticks = ([1, 2, 3, 4, 5],
                   [1, 2, 3, 4, 5],
                   [2, 4, 6, 8, 10],)
    labels = (['$\mathrm{(a)}$', '$\mathrm{(b)}$'], ['$\mathrm{(c)}$',
              '$\mathrm{(d)}$'], ['$\mathrm{(e)}$', '$\mathrm{(f)}$'])
    
    for i, (key, gs) in enumerate(zip(keys, gammas)):
        delta = deltas[key]
        # Dispersion
        ax1 = fig.add_subplot(3, 2, 2*i+1)
        ks = disp_data[key][0]
        sc_disp = disp_data[key][1]
        N_disp = disp_data[key][2]
        subplot_disp(ax1, sc_disp, ks*10, N_disp=N_disp, title='',
                     xlim=(0, max(ks*10)), ylim=(0, 1.05*delta/delta),
                     yticks=[0, 0.5, 1],
                     xticks=disp_xticks[i], label=labels[i][0], xlabel=r'$k \ [\mathrm{nm}^{-1}]$')
        # DOS
        ax2 = fig.add_subplot(3, 2, 2*i+2)
        subplot_DOS(ax2, dos_data[key][1], dos_data[key][0], delta, Elim=[0, 1.05*delta],
                    DOSlim=[0, 40], title='', gammas=gs, label=labels[i][1])

    plt.tight_layout()
    plt.subplots_adjust(wspace=0.05, hspace=0.28)
    plt.savefig(name, bbox_inches='tight')
    plt.show()


# In[27]:

disp_dos_plot()


# In[ ]:



