"""Module of functions used to compute data for the paper "The Andreev
rectifier: a nonlocal conductance signature of topological phase transitions"
by Tómas Örn Rosdahl, Adriaan Vuik, Morten Kjærgaard and Anton Akhmerov.

arXiv:1706.08888."""

import kwant
import numpy as np
from scipy import sparse
import constants as const
import tools as tls

from kwant.digest import uniform


class GateParameters(object):
    """Container to store gate potential parameters"""
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


class DimensionParameters(object):
    """Container to store dimension parameters"""
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

# Onsite disorder.
disorder = (lambda site, salt: (2 * uniform(str(site.tag), salt) - 1))

# Tunnel gate potentials
tunnel_pots = (lambda site, dim_par, gate_par:
               tunnel_gates(site.pos[0], site.pos[1], dim_par, gate_par))

# Gaussian potential
gauss_potential = (lambda site, V_gauss, dim_par:
                   gaussian_2D(site.pos[0], site.pos[1], V_gauss, dim_par))


def make_inf_lateral_sys(W_N, W_sc):
    """"Construct a one-dimensional system that is partially normal and
    partially superconducting. The junction is translationally invariant
    along the interface between the normal and superconducting parts.

        Parameters:
        ------------------
        W_N: int
            Width of the normal part, in units of the lattice constant.
        W_sc: int
            Width of the superconducting part, in units of the lattice
            constant.
        """

    # Define Hamiltonian and kwant lattice
    pauli_matrices = ["sigma","tau"] # spin, electron-hole pseudospin
    bdg_ham = ("((x_scale*T*p_x**2 + y_scale*T*p_y**2 + V - mu) + \
               alpha * (y_scale*p_y*sigma_x - x_scale*p_x*sigma_y)) * tau_z + \
               Ez * sigma_x + delta * tau_x")

    lat = kwant.lattice.square(1)
    sys = kwant.Builder(kwant.TranslationalSymmetry((-1,0)))

    def sub_int(W_lower, W_upper):
        def bool_sub(pos):
            (x, y) = pos
            return W_lower <= y < W_upper
        return bool_sub, (0, W_lower)

    # Find hoppings between the superconductor and the normal part.
    def at_sc_interface(site1, site2):
        return ((sc_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and sc_shape[0](site2.pos)))
    # Find hoppings in the superconductor.
    def in_sc_scatter(site1, site2):
        return (sc_shape[0](site1.pos) and sc_shape[0](site2.pos))

    # Replacements for various regions
    no_sc = "delta = 0"
    no_pot = "V = 0"
    # Scaled hopping/onsites along x
    x_not_scaled = "x_scale = 1.0"
    # Scaled hopping/onsite along y
    y_not_scaled = "y_scale = 1.0"
    no_mag_field = "Ez = 0"

    normal_shape = sub_int(0, W_N)
    sc_shape = sub_int(W_N, W_N+W_sc)

    # Onsite - the normal part.
    if W_N > 0:
        onsite = tls.make_tb_system(bdg_ham,
                                    [no_sc, no_pot, x_not_scaled, y_not_scaled],
                                    pauli_matrices, dim=2)[0]
        sys[lat.shape(*normal_shape)] = onsite

    # Onsite - the superconducting part.
    if W_sc > 0:
        onsite = tls.make_tb_system(bdg_ham,
                                    [no_mag_field, no_pot, y_not_scaled],
                                    pauli_matrices, dim=2)[0]
        sys[lat.shape(*sc_shape)] = onsite

    # Hoppings
    hops = tls.make_tb_system(bdg_ham,
                              [x_not_scaled, y_not_scaled],
                              pauli_matrices, dim=2)[1]
    sc_hops = tls.make_tb_system(bdg_ham,
                                 [y_not_scaled], pauli_matrices, dim=2)[1]
    # Modified hoppings at the junction between the normal part and the SC.
    sc_interface_hops = tls.make_tb_system(bdg_ham,
                                           [x_not_scaled],
                                           pauli_matrices, dim=2)[1]

    for kind, value in list(hops.items()):
        sys[kwant.builder.HoppingKind(kind, lat)] = value

    # Replace hoppings in the superconducting part.
    for kind, value in sc_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if in_sc_scatter(i, j)
            )
        sys[hopping_iterator] = value

    # Hoppings between normal part and superconductor.
    if (W_sc > 0) and (W_N > 0):
        for kind, value in sc_interface_hops.items():
            hopping_iterator = (
                (i, j)
                for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
                if at_sc_interface(i, j)
                )
            sys[hopping_iterator] = value
    sys = sys.finalized()
    return sys


def make_inf_normal_sys(W_N):
    """"Construct a one-dimensional normal system.

        Parameters:
        ------------------
        W_N: int
            Width of the normal part, in units of the lattice constant.
        """

    # Define Hamiltonian and kwant lattice
    pauli_matrices = ["sigma"] # spin
    ham = ("(T*p_x**2 + T*p_y**2 - mu) + alpha*(p_y*sigma_x - p_x*sigma_y) + \
            Ez*sigma_x")
    lat = kwant.lattice.square(1)
    sys = kwant.Builder(kwant.TranslationalSymmetry((-1,0)))

    def sub_int(W_lower, W_upper):
        def bool_sub(pos):
            (x, y) = pos
            return W_lower <= y < W_upper
        return bool_sub, (0, W_lower)

    normal_shape = sub_int(0, W_N)
    # Onsites
    onsite = tls.make_tb_system(ham, [], pauli_matrices, dim=2)[0]
    sys[lat.shape(*normal_shape)] = onsite
    # Hoppings
    hops = tls.make_tb_system(ham, [], pauli_matrices, dim=2)[1]
    for kind, value in list(hops.items()):
        sys[kwant.builder.HoppingKind(kind, lat)] = value
    sys = sys.finalized()
    return sys


def get_bands(k, c, W_N, W_sc):
    """Compute the band structure of a translationaly invariant
       normal-superconductor junction at a single momentum.

       Parameters:
       -----------------
       k: float
          Momentum at which to compute the band structure, in units
          the lattice constant.
       c: tools.SimpleNamespace object
          A container with all Hamiltonian parameters.
       W_N: float
          Width of the normal part in physical units.
       W_sc: float
          Width of the superconducting part in physical units.
    """
    sys = make_inf_lateral_sys(d_sys(W_N, c.a), d_sys(W_sc, c.a))
    bands = kwant.physics.Bands(sys, args=[c])
    return bands(k)


def get_bands_normal(k, c, W_N):
    """Compute the band structure of a translationaly invariant
       normal system at a single momentum.

       Parameters:
       -----------------
       k: float
          Momentum at which to compute the band structure, in units
          the lattice constant.
       c: tools.SimpleNamespace object
          A container with all Hamiltonian parameters.
       W_N: float
          Width of the normal part in physical units.
    """
    sys = make_inf_normal_sys(d_sys(W_N, c.a))
    bands = kwant.physics.Bands(sys, args=[c])
    return bands(k)


def DOS_1D_vel(lead, energy=0, args=[]):
    """Compute the density of states of a translationally invariant
       system at a given energy.

       Parameters:
       -------------------
       lead: kwant.system.InfiniteSystem object
             A finalized Kwant 1D translationally invariant system.
       energy: float
             Energy at which to compute the density of states.
       args: iterable
             Iterable of parameters that are passed on to
             the Kwant system construction function.
    """
    vel = lead.modes(energy=energy, args=args)[0].velocities
    # No states at this energy
    if len(vel) == 0:
        return 0
    # Return sum of absolute values of inverted velocities
    else:
        return sum(np.abs(1/vel))


def get_nmodes(lead, c, E=0):
    """Compute the number of propagating modes for a 1D translationally
    invariant system at a given energy.

    Parameters:
    -------------------
    lead: kwant.system.InfiniteSystem object
          A finalized Kwant 1D translationally invariant system.
    c: tools.SimpleNamespace object
          A container with all Hamiltonian parameters.
    E: float
          Energy at which to compute the density of states.
    """
    return lead.modes(energy=E, args=[c])[1].nmodes


def gap_finder(lead, c, start_Es=(0, 1.0), eps=1e-10):
    """Compute the induced gap from the dispersion of a lead by finding the
    smallest energy at which there are propagating modes. Positive energies
    are assumed.

    Parameters:
    -------------------
    lead: kwant.system.InfiniteSystem object
          A finalized Kwant 1D translationally invariant system.
    c: tools.SimpleNamespace object
          A container with all Hamiltonian parameters.
    start_Es: tuple of two floats
          Starting guess for an energy interval around the induced gap.
          It is assumed that start_Es[1] > start_Es[0].
    eps: float
          Tolerance for the induced gap. """

    E0, E1 = start_Es
    nmodes0 = get_nmodes(lead, c, E=E0)
    nmodes1 = get_nmodes(lead, c, E=E1)
    if not (nmodes0 == 0 and nmodes1 != 0):
        raise ValueError('Give initial energies such that modes are \
            active at one but not the other')
    while True:
        new_E = 0.5*(E1+E0)
        new_nmodes = get_nmodes(lead, c, E=new_E)
        if new_nmodes == 0:
            previous_E = E0
            E0 = new_E
        elif new_nmodes != 0:
            previous_E = E1
            E1 = new_E
        if np.abs(new_E - previous_E) < eps:
            break
    return new_E


def DOS_parallel(E, c, W_N, W_sc, a):
    """Compute the density of states of a translationally invariant
       normal-superconductor junction at a single energy.

       Parameters:
       -------------------
       E: float
          Energy at which to compute the density of states.
       c: tools.SimpleNamespace object
          A container with all Hamiltonian parameters.
       W_N: float
          Width of the normal part in physical units.
       W_sc: float
          Width of the superconducting part in physical units.
       a: float
          Lattice constant.
    """
    sys = make_inf_lateral_sys(d_sys(W_N, a), d_sys(W_sc, a))
    return DOS_1D_vel(sys, energy=E, args=[c])


def gaussian(x, x0, sigma):
    "Gaussian centered at x0 with standard deviation sigma. "
    return np.exp(-0.5*((x - x0)/sigma)**2)


def gaussian_2D(x, y, V0, dim_par):
    "Product of two Gaussians, centered at (x0, y0)"
    x0 = dim_par.L/2.
    y0 = dim_par.W_L/2.
    std_x = dim_par.L/5.
    std_y = dim_par.W_L/1.5
    return V0*gaussian(x, x0, std_x)*gaussian(y, y0, std_y)


def Transverse_Thouless(W_N, T, a, t=1.0):
    """Compute transverse Thouless energy E_T in meV of a 1D
    normal-superconductor junction with transparency t. Quasiparticle
    lifetime in the normal part is estimated using the zero point motion of
    the lowest transverse mode in an infinite square well to approximate
    the normal part of the junction.

    Parameters:
    ------
    W_N: float
        width of the normal part in physical units.
    T: float
        hbar**2/2ma**2, where a is the
        lattice constant and m the effective mass.
    a: float
        lattice constant in nm.
    t: float
        transparency of the normal-superconductor interface.
    """
    return t*T*np.pi**2/(2*W_N/a)**2


def interface_trans(k, mu, T, E=0, alpha=0, branch='+'):
    """Compute the transparency of the interface between N and SC
    at momentum k parallel to the interface, at excitation energy E,
    i.e. at the chemical potential. No hopping is assumed along the
    interface in the superconductor. See Phys. Rev. B 95, 115421 (2017).

    Parameters:
    ------
    k: float
        momentum parallel to the interface in units of the lattice constant.
    mu: float
        chemical potential of the system in meV.
    T: float
        hbar**2/2ma**2 in meV, where a is the lattice constant and m the
        effective mass in the normal part, and normal to the interface in SC.
    E: float
        energy in meV
    alpha: float
        SOI strength, scaled by the lattice constant (i.e. alpha/a with alpha
        in meVnm and a lattice constant in nm)
    branch: '+' or '-'
        Specifies which spin branch of the 1D RSOI dispersion to use in the
        normal part.
    """
    q = np.sqrt((E+mu)/T)
    if branch is '+':
        k = np.sqrt((E+mu+alpha*np.abs(k))/T - k**2)
        trans = 4*(np.sqrt(k/q) + np.sqrt(q/k))**(-2)
    elif branch is '-':
        k = np.sqrt((E+mu-alpha*np.abs(k))/T - k**2)
        trans = 4*(np.sqrt(k/q) + np.sqrt(q/k))**(-2)
    return trans


def kxF_trans(mu, offset, T, E=0, alpha=0, branch='+'):
    """ Compute the Fermi momentum parallel to the interface in a 1D
    normal-superconductor junction. The momentum associated with the
    lowest electron mode at the chemical potential is used. Excitation
    energy E [meV] and no Zeeman splitting are assumed.
    See Phys. Rev. B 95, 115421 (2017).

    Parameters:
    ------
    mu: float
        chemical potential in meV
    offset: float
        band offset of the lowest electron mode in the normal part.
    T: float
        hbar**2/2ma**2 in meV, where a is the lattice constant and m the
        effective mass in the normal part, and normal to the interface in
        the superconductor.
    E: float
        energy in meV.
    alpha: float
        SOI strength, scaled by the lattice constant (i.e. alpha/a with
        alpha in meVnm and a lattice constant in nm)
    branch: '+' or '-'
        Specifies which spin branch of the 1D RSOI dispersion to use in
        the normal part.

    Returns:
    -------
    kF: Fermi momentum, scaled in terms of the lattice constant."""

    if branch is '+':
        return 0.5*(alpha/T + np.sqrt((alpha/T)**2 - 4*(offset-mu-E)/T))
    elif branch is '-':
        return 0.5*(-alpha/T + np.sqrt((alpha/T)**2 - 4*(offset-mu-E)/T))


def modes(h_cell, h_hop, tol=1e6):
    """Compute the eigendecomposition of a translation operator of a lead.

    Adapted from kwant.physics.leads.modes such that it returns the
    eigenvalues.

    Parameters:
    ----------
    h_cell : numpy array, real or complex, shape (N, N) The unit cell
        Hamiltonian of the lead unit cell.
    h_hop : numpy array, real or complex, shape (N, M)
        The hopping matrix from a lead cell to the one on which self-energy
        has to be calculated (and any other hopping in the same direction).
    tol : float
        Numbers and differences are considered zero when they are smaller
        than `tol` times the machine precision.

    Returns
    -------
    ev : numpy array
        Eigenvalues of the translation operator in the form lambda=exp(i*k).
    """
    m = h_hop.shape[1]
    n = h_cell.shape[0]

    if (h_cell.shape[0] != h_cell.shape[1] or
            h_cell.shape[0] != h_hop.shape[0]):
        raise ValueError("Incompatible matrix sizes for h_cell and h_hop.")

    # Note: np.any(h_hop) returns (at least from numpy 1.6.1 - 1.8-devel)
    #       False if h_hop is purely imaginary
    if not (np.any(h_hop.real) or np.any(h_hop.imag)):
        v = np.empty((0, m))
        return (kwant.physics.PropagatingModes(np.empty((0, n)), np.empty((0,)),
                                               np.empty((0,))),
                kwant.physics.StabilizedModes(np.empty((0, 0)),
                                              np.empty((0, 0)), 0, v))

    # Defer most of the calculation to helper routines.
    matrices, v, extract = kwant.physics.leads.setup_linsys(
        h_cell, h_hop, tol, None)
    ev = kwant.physics.leads.unified_eigenproblem(*(matrices + (tol,)))[0]
    return ev


def slowest_evan_mode(lead, p):
    """Find the slowest decaying (evanescent) mode.

    It uses an adapted version of the function kwant.physics.leads.modes,
    in such a way that it returns the eigenvalues of the translation operator
    (lamdba = e^ik). The imaginary part of the wavevector k, is the part that
    makes it decay. The inverse of this Im(k) is the size of a Majorana bound
    state. The norm of the eigenvalue that is closest to one is the slowest
    decaying mode. Also called decay length.

    Parameters:
    -----------
    lead : kwant.builder.InfiniteSystem object
        The finalized infinite system.
    p : tools.SimpleNamespace object
        A simple container that is used to store Hamiltonian parameters.

    Returns:
    --------
    majorana_length : float
        The length of the Majorana.
    """
    h, t = lead.cell_hamiltonian(args=[p]), lead.inter_cell_hopping(args=[p])
    ev = modes(h, t)
    # index of the slowest mode by closenest to the unit circle
    idx = np.abs(np.abs(ev) - 1).argmin()

#     majorana_length = np.abs(a / np.log(ev[idx]).real)
    majorana_length = np.abs(p.a / np.log(np.abs(ev[idx])))
    return majorana_length


def decay_length(W_N, W_sc, c):
    """Compute decay length of the slowest decaying mode for
    a translationally invariant normal-superconductor junction.

    Parameters:
    -----------
    W_N: float
        Width of the normal part in physical units.
    W_sc: float
        Width of the superconducting part in physical units.
    c: tools.SimpleNamespace object
        A simple container that is used to store Hamiltonian parameters.
    """
    lead = make_inf_lateral_sys(d_sys(W_N, c.a), d_sys(W_sc, c.a))
    decay_length = slowest_evan_mode(lead, c)
    return decay_length


def d_sys(D, a):
    """A helper function to scale a dimension D in physical units correctly
       in terms of a lattice constant a.
       """
    return int(np.round((D-a)/a))


def get_cond(SMat, lead_out, lead_in, trans='nonloc'):
    """Compute various transmission reflection processes in a
    normal-superconductor or a normal-superconductor-normal junction.
       Parameters:
       ----------
        SMat: kwant.solvers.common.SMatrix
            Scattering matrix of the system.
        lead_out: integer
            Index of the out lead.
        lead_in: integer
            Index of the in lead.
        trans: string
            Specifies which process to compute (see cases below).
    """
    if trans == 'nonloc':
        # Nonlocal electron to electron and electron to hole contributions
        data = SMat.transmission((lead_out, 0), (lead_in, 0)) - \
               SMat.transmission((lead_out, 1), (lead_in, 0))
    elif trans == 'the':
        # Transmission, electron to hole.
        data = SMat.transmission((lead_out, 1), (lead_in, 0))
    elif trans == 'tee':
        # Transmission, electron to electron.
        data = SMat.transmission((lead_out, 0), (lead_in, 0))
    elif trans == 'loc':
        # Local electron to electron and electron to hole contributions
        data = (SMat.submatrix((lead_out, 0), (lead_in, 0)).shape[0] - \
                SMat.transmission((lead_out, 0), (lead_in, 0)) + \
                SMat.transmission((lead_out, 1), (lead_in, 0)))
    elif trans == 'ree':
    # Only electron to electron reflection part
        data = SMat.transmission((lead_out, 0), (lead_in, 0))
    elif trans == 'rhe':
    # Only electron to hole reflection part
        data = SMat.transmission((lead_out, 1), (lead_in, 0))
    elif trans == 'all':
    # Total two-terminal conductance as returned by Kwant, not
    # accounting for quasiparticle charge.
        data = SMat.transmission(lead_out, lead_in)
    return data


def get_band_min(W, T, mode_num):
    """ Compute the band offset of a transverse mode in a normal junction of
    finite width.

    Parameters:
    ---------
    W: int
        Width of the junction in units of the lattice constant.
    T: float
        hbar**2/2ma**2 in meV, where a is the lattice constant and m the
        effective mass.
    mode_num: int >= 1
        Index of the band.
    """
    return T*np.pi**2*mode_num**2/W**2


def mean_free_path(E_F, c):
    """ Compute the mean free path.
    Parameters:
    -----------
    E_F: float
        Fermi energy in meV at which the mean free path is computed.
    c: tools.SimpleNamespace object
        A simple container that is used to store Hamiltonian parameters.
    Returns:
    -----------
    l_e: float
        Mean free path in units of the lattice constant.

    Note: We assume disorder potentials in the range [-V:V]."""
    return 48*np.sqrt(E_F/c.T)*(c.T/(c.dis*2))**2


def dis_from_mfp(l_e, E_F, T):
    """ Compute disorder potential amplitude from a given mean free path.
    Parameters:
    -----------
    l_e: float or int
        mean free path in units of the lattice constant.
    E_F: float
        The Fermi energy in meV.
    T: float
        hbar**2/2ma**2 in meV, where a is the lattice constant and m the
        effective mass.
    Returns:
    --------
    dis: float
        The disorder amplitude in meV corresponding to the mean free path.
        """
    return T*np.sqrt(12.0/l_e * np.sqrt(E_F/T))


def rectangle_potential(x, y, L, R, B, T, V_G, d):
    """Potential profile due to a rectangular gate with voltage V_G, that
    extends from L < x < R and B < y < T and is located a distance d
    below the 2DEG."""

    def g(u, v):
        R = np.sqrt(u**2 + v**2 + d**2)
        return 1./2./np.pi*np.arctan2(u*v, d*R)

    phi = V_G * (g(x - L, y - B) + g(x - L, T - y) + \
                 g(R - x, y - B) + g(R - x, T - y))

    return phi


def determinant(Ez, c, dim_par, lead_in):
    """ Compute the determinant of the reflection matrix for a given
    Zeeman field. """
    sys = make_NSN_tunnel_gates_gauss(dim_par)
    c.Ez = Ez
    smatrix = kwant.smatrix(sys, 0.0, args = [c])
    r = smatrix.submatrix(lead_in, lead_in)
    return abs(np.linalg.det(r))


def diag_hamiltonian(Ez, c, dim_par, lead_out, lead_in, return_eigvecs=False):
    """Compute eigen energies of the Hamiltonian of a closed system as a
    function of Zeeman field"""
    c.Ez = Ez
    c.dim_par = dim_par
    finite_sys = make_NS_closed_sys_gauss_pot(dim_par, plot_sys=False)
    ham_mat = finite_sys.hamiltonian_submatrix(sparse=True, args=[c])
    sigma = 0
    eigen_energies, wave_modes = sparse.linalg.eigsh(ham_mat, k=20,
                                                     sigma=sigma, which='LM')
    if return_eigvecs:
        return eigen_energies, wave_modes
    else:
        return eigen_energies


def make_NSN_tunnel_gates(dim_par, plot_sys=False, sc_chains=False):
    """Build the NSN system with two normal leads (left and right) and one
    superconducting lead (top). The normal leads are separated into electron
    and hole blocks.
    The leads are attached in the following order:
    left lead            --> lead 0
    right lead           --> lead 1
    superconducting lead --> lead 2.
    Tunnel gates (QPCs) for the left and right leads.

    Parameters:
    ----------
    dim_par: tools.SimpleNamespace object
        A container that is used to store the dimension parameters of the
        system.
    plot_sys: bool
        Specify whether to plot the system or not.
    sc_chains: bool
        Specify whether to treat the superconductor as a sequence of
        uncoupled chains, or treat it as a single lead.
    """
    # Extract the relevant dimensions
    L = dim_par.L  # Length.
    dL = dim_par.dL  # Length of left/right leads in scattering region.
    W_L = dim_par.W_L  # Width of left/right normal leads.
    dW_ML = dim_par.dW_ML  # Extra width of the normal scattering region.
    dW_SC = dim_par.dW_SC  # Width of superconductor part in scattering region.

    # Define Hamiltonian and kwant lattice
    pauli_matrices = ["sigma","tau"] # spin, electron-hole
    bdg_ham = ("((x_scale*T*p_x**2 + y_scale*T*p_y**2 + V - mu) + \
               alpha * (y_scale*p_y*sigma_x - x_scale*p_x*sigma_y)) * tau_z + \
               Ez * sigma_x + delta * tau_x")
    lat = kwant.lattice.square(a=1, norbs=4)
    sys = kwant.Builder()

    def rectangle(L_left, L_right, W_lower, W_upper):
        def bool_rect(pos):
            (x, y) = pos
            return (L_left <= x < L_right) and (W_lower <= y < W_upper)
        return bool_rect, (L_left, W_lower)

    # Pick out hoppings between the normal part and the superconductor.
    def at_sc_interface(site1, site2):
        return ((sc_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and sc_shape[0](site2.pos)))
    # Pick out hoppings in the superconductor.
    def in_sc_scatter(site1, site2):
        return (sc_shape[0](site1.pos) and sc_shape[0](site2.pos))

    # Replacements for various regions
    no_sc = "delta = 0"
    x_not_scaled = "x_scale = 1.0"
    y_not_scaled = "y_scale = 1.0"
    gate_and_disorder = "V = dis * disorder(site, salt) - \
                         tunnel_pots(site, dim_par, gate_par)"
    gates = "V = -tunnel_pots(site, dim_par, gate_par)"
    no_pot = "V = 0"
    mu_sc = "mu = mu_sc"
    no_mag_field = "Ez = 0"

    #### Construct the scattering region.
    normal_shape = rectangle(dL, L+dL, dW_ML, dW_ML+W_L)
    sc_shape = rectangle(dL, L+dL, dW_ML+W_L, dW_ML+W_L+dW_SC)
    ml_shape = rectangle(dL, L+dL, 0, dW_ML)
    if dL != 0:
        ll_shape = rectangle(0, dL, dW_ML, dW_ML+W_L)
        rl_shape = rectangle(L+dL, L+2*dL, dW_ML, dW_ML+W_L)

    # Onsite - normal part.
    onsite = tls.make_tb_system(bdg_ham,
        [no_sc, mu_sc, gate_and_disorder, x_not_scaled, y_not_scaled],
        pauli_matrices, dim=2)[0]
    sys[lat.shape(*normal_shape)] = onsite
    # Onsite - superconductor
    onsite = tls.make_tb_system(bdg_ham,
        [mu_sc, gate_and_disorder, no_mag_field, y_not_scaled],
        pauli_matrices, dim=2)[0]
    sys[lat.shape(*sc_shape)] = onsite
    # Onsite - the normal lead parts of the scattering region.
    if dW_ML > 0:
        onsite = tls.make_tb_system(bdg_ham,
            [no_sc, mu_sc, gate_and_disorder, x_not_scaled, y_not_scaled],
            pauli_matrices, dim=2)[0]
        sys[lat.shape(*ml_shape)] = onsite
    if dL != 0:
        onsite = tls.make_tb_system(bdg_ham,
            [no_sc, gates, x_not_scaled, y_not_scaled],
            pauli_matrices, dim=2)[0]
        sys[lat.shape(*ll_shape)] = onsite
        onsite = tls.make_tb_system(bdg_ham,
            [no_sc, gates, x_not_scaled, y_not_scaled],
            pauli_matrices, dim=2)[0]
        sys[lat.shape(*rl_shape)] = onsite

    # Hoppings
    hops = tls.make_tb_system(bdg_ham, [x_not_scaled, y_not_scaled],
                              pauli_matrices, dim=2)[1]
    # Hoppings in the superconductor
    sc_hops = tls.make_tb_system(bdg_ham, [y_not_scaled],
                                 pauli_matrices, dim=2)[1]
    # Hoppings at the interface between normal and superconductor
    sc_interface_hops = tls.make_tb_system(bdg_ham, [x_not_scaled],
                                           pauli_matrices, dim=2)[1]

    for kind, value in hops.items():
        sys[kwant.builder.HoppingKind(kind, lat)] = value

    for kind, value in sc_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if in_sc_scatter(i, j)
            )
        sys[hopping_iterator] = value

    for kind, value in sc_interface_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if at_sc_interface(i, j)
            )
        sys[hopping_iterator] = value

    #### Add leads.
    def lr_lead_shape(pos):
        (x, y) = pos
        return (dW_ML <= y < dW_ML+W_L)

    # Normal leads (left and right)
    # Particle-hole degree of freedom is conserved.
    # Partition the scattering matrix into electron and hole blocks.
    PH_cons = np.kron(np.eye(2), -const.sz)
    left_lead = kwant.Builder(kwant.TranslationalSymmetry((-1, 0)),
                              conservation_law=PH_cons)
    right_lead = kwant.Builder(kwant.TranslationalSymmetry((1, 0)),
                               conservation_law=PH_cons)
    onsite, hops = tls.make_tb_system(bdg_ham,
        [no_sc, no_pot, x_not_scaled, y_not_scaled],
        pauli_matrices, dim=2)
    left_lead[lat.shape(lr_lead_shape, (-1, dW_ML))] = onsite
    onsite, hops = tls.make_tb_system(bdg_ham,
        [no_sc, no_pot, x_not_scaled, y_not_scaled],
        pauli_matrices, dim=2)
    right_lead[lat.shape(lr_lead_shape, (L+2*dL, dW_ML))] = onsite
    for kind, value in hops.items():
        left_lead[kwant.builder.HoppingKind(kind, lat)] = value
        right_lead[kwant.builder.HoppingKind(kind, lat)] = value
    sys.attach_lead(left_lead)
    sys.attach_lead(right_lead)

    # Superconducting lead
    if not sc_chains:
        # We treat the superconductor as a single lead
        def middle_lead_shape(pos):
            (x, y) = pos
            return (dL <= x < L+dL)
        sc_lead = kwant.Builder(kwant.TranslationalSymmetry((0,1)))
        onsite, hops = tls.make_tb_system(bdg_ham,
            [mu_sc, no_mag_field, no_pot, y_not_scaled],
            pauli_matrices, dim=2)
        sc_lead[lat.shape(middle_lead_shape, (dL, dW_ML+W_L+dW_SC))] = onsite
        for kind, value in hops.items():
            sc_lead[kwant.builder.HoppingKind(kind, lat)] = value
        sys.attach_lead(sc_lead)
    else:
        # Treat the superconductor as multiple 1D leads attached in succession
        for i in range(dL, L+dL):
            def middle_lead_shape(pos):
                (x, y) = pos
                return x == i
            # Scaled hopping/onsites along x - here, it must be set to 0!
            zero_x_hopping = "x_scale = 0.0"
            sc_lead = kwant.Builder(kwant.TranslationalSymmetry((0,1)))
            onsite, hops = tls.make_tb_system(bdg_ham,
                [mu_sc, no_mag_field, no_pot, y_not_scaled, zero_x_hopping],
                pauli_matrices, dim=2)
            sc_lead[lat.shape(middle_lead_shape, (i, dW_ML+W_L+dW_SC))] = onsite
            for kind, value in hops.items():
                sc_lead[kwant.builder.HoppingKind(kind, lat)] = value
            sys.attach_lead(sc_lead)
    if plot_sys:
        kwant.plot(sys)
    sys = sys.finalized()
    return sys


def tunnel_gates(x, y, dim_par, gate_par):
    """Potential for tunnel barriers (left and right leads) from Sukhorukov's
    analytic expressions. The origin (0, 0) is the left lower
    corner of the scattering region. Gate lengths >> scattering
    region to avoid boundary effects.

    Parameters:
    ----------
    dim_par: tools.SimpleNamespace object
        Container for the dimension parameters of the tunnel gates.
    gate_par: tools.SimpleNamespace object
        Container for gate voltages on tunnel gates.

    The dimension parameters are (in units of the lattice constant):
    W_L: Width of the normal leads
    dW_ML: Extra width of the normal part of the scattering region,
        such that the total width is W_L+dW_ML.
    dW_SC: Width of a segment of the superconductor to include in the
        scattering region.
    dL: Lengths of segments of the normal leads to include in the scattering
        region.
    d: The distance that a gate is placed under the 2DEG.
    dgT: Distance from the central part of the scattering region to
        the gates at the junctions with the normal leads.
    wgt: Width of the gates at the junctions with the normal leads.
    dW_T: The distance the gates extend into the normal leads
          (from top and bottom).

    The gate parameters are:
    V_gtL: voltage on the gates by the junction with the left lead.
    V_gtR: voltage on the gates by the junction with the right lead.
    """
    # Extract all the geometry dimensions.
    W_L = dim_par.W_L
    dW_ML = dim_par.dW_ML
    dL = dim_par.dL
    L = dim_par.L
    d = dim_par.d
    dgT = dim_par.dgT
    wgt = dim_par.wgt
    dW_T = dim_par.dW_T

    # Extract all the gate voltages
    V_gtL = gate_par.V_gtL
    V_gtR = gate_par.V_gtR

    # Make the side-gate potentials that create tunnel barriers at the borders
    # Arguments are (x, y, left, right, bottom, top)
    left = dL - dgT - wgt
    right = dL - dgT
    bottom = dW_ML + W_L - 1 - dW_T
    top = 1000
    tunnel_gate_top_left = rectangle_potential(x, y, left, right,
                                               bottom, top, V_gtL, d)
    bottom = -1000
    top = dW_ML + dW_T
    tunnel_gate_bottom_left = rectangle_potential(x, y, left, right,
                                                  bottom, top, V_gtL, d)
    left = dL + L + dgT - 1
    right = dL + L + dgT + wgt - 1
    tunnel_gate_bottom_right = rectangle_potential(x, y, left, right,
                                                   bottom, top, V_gtR, d)
    bottom = dW_ML + W_L - 1 - dW_T
    top = 1000
    tunnel_gate_top_right = rectangle_potential(x, y, left, right,
                                                bottom, top, V_gtR, d)

    total_potential = (tunnel_gate_top_left + tunnel_gate_top_right +
                       tunnel_gate_bottom_left + tunnel_gate_bottom_right)
    return total_potential


def make_NS_closed_sys_gauss_pot(dim_par, plot_sys=False):
    """Make a closed rectangular NS system to compute the energy spectrum.
    A Gaussian potential is included in the center of the system to model
    a long-range potential inhomogeneity."""
    # Extract the relevant dimensions
    L = dim_par.L  # Length
    W_L = dim_par.W_L  # Width of left/right normal leads.
    W_SC = dim_par.W_SC # Width of superconductor in the normal region.

    # Define Hamiltonian and kwant lattice
    pauli_matrices = ["sigma","tau"] # spin, electron-hole pseudospin
    bdg_ham = ("((x_scale*T*p_x**2 + T*p_y**2 + V - mu) + \
                alpha * (p_y * sigma_x - p_x * sigma_y)) * tau_z + \
                Ez * sigma_x + delta * tau_x")
    lat = kwant.lattice.square(1)
    sys = kwant.Builder()

    def rectangle(L_left, L_right, W_lower, W_upper):
        def bool_rect(pos):
            (x, y) = pos
            return (L_left <= x < L_right) and (W_lower <= y < W_upper)
        return bool_rect, (L_left, W_lower)

    def at_sc_interface(site1, site2):
        return ((sc_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and sc_shape[0](site2.pos)))

    def in_sc_scatter(site1, site2):
        return (sc_shape[0](site1.pos) and sc_shape[0](site2.pos))

    # Replacements for various regions
    no_sc = "delta = 0"
    x_not_scaled = "x_scale = 1.0"
    sc_interface = "T = T_sc"
    mu_sc = "mu = mu_sc"
    no_mag_field = "Ez = 0"
    no_pot = "V = 0"
    gauss_pot = "V = -gauss_potential(site, V_gauss, dim_par)"

    #### Construct the scattering region.
    # Central part of the scattering region.
    normal_shape = rectangle(0, L, 0, W_L)
    # Following are parts of the scattering region, but directly attached
    # to one of the four leads.
    sc_shape = rectangle(0, L, W_L, W_L+W_SC)

    # Onsite - the central normal part.
    onsite = tls.make_tb_system(bdg_ham, [no_sc, x_not_scaled, gauss_pot],
                                pauli_matrices, dim=2)[0]
    sys[lat.shape(*normal_shape)] = onsite
    # Onsite - the superconducting part.
    onsite = tls.make_tb_system(bdg_ham, [mu_sc, no_mag_field, no_pot],
                                pauli_matrices, dim=2)[0]
    sys[lat.shape(*sc_shape)] = onsite
    # Onsite - the normal lead parts.


    # Unchanged hoppings in entire scattering region.
    hops = tls.make_tb_system(bdg_ham, [x_not_scaled], pauli_matrices,
                              dim=2)[1]
    # Hoppings along x scaled in the superconducting part.
    sc_hops = tls.make_tb_system(bdg_ham, [], pauli_matrices, dim=2)[1]
    # Modified hoppings at the junction between the normal part and the SC.
    sc_interface_hops = tls.make_tb_system(bdg_ham, [sc_interface],
                                           pauli_matrices, dim=2)[1]

    for kind, value in hops.items():
        sys[kwant.builder.HoppingKind(kind, lat)] = value

    # Replace hoppings in the superconducting part of the scattering region.
    # Here we use scaled hoppings along x.
    for kind, value in sc_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if in_sc_scatter(i, j)
            )
        sys[hopping_iterator] = value

    # Hoppings between normal part and superconductor. These hoppings
    # are along y.
    for kind, value in sc_interface_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if at_sc_interface(i, j)
            )
        sys[hopping_iterator] = value
        ########

    if plot_sys:
        kwant.plot(sys)
    return sys.finalized()


def make_NSN_tunnel_gates_gauss(dim_par, plot_sys=False, sc_chains=False):
    """Build the NSN system with two normal leads (left and right) and
    one superconducting lead (top). Make the leads particle-hole symmetric.
    The leads are attached in the following order:
    left lead            --> lead 0
    right lead           --> lead 1
    superconducting lead --> lead 2.
    Tunnel gates (QPCs) for the left and right leads.
    A Gaussian potential is included in the center of the system to model
    a long-range potential inhomogeneity.
    """
    # Extract the relevant dimensions
    L = dim_par.L  # Length
    dL = dim_par.dL  # Length of normal leads in scattering region.
    W_L = dim_par.W_L  # Width of left/right normal leads.
    dW_ML = dim_par.dW_ML
    dW_SC = dim_par.dW_SC

    # Define Hamiltonian and kwant lattice
    pauli_matrices = ["sigma","tau"] # spin, electron-hole pseudospin
    bdg_ham = ("((x_scale*T*p_x**2 + y_scale*T*p_y**2 + V - mu) + \
               alpha * (y_scale*p_y*sigma_x - x_scale*p_x*sigma_y)) * tau_z + \
               Ez * sigma_x + delta * tau_x")
    lat = kwant.lattice.square(a=1, norbs=4)
    sys = kwant.Builder()

    def rectangle(L_left, L_right, W_lower, W_upper):
        def bool_rect(pos):
            (x, y) = pos
            return (L_left <= x < L_right) and (W_lower <= y < W_upper)
        return bool_rect, (L_left, W_lower)

    def at_sc_interface(site1, site2):
        return ((sc_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and sc_shape[0](site2.pos)))

    def in_sc_scatter(site1, site2):
        return (sc_shape[0](site1.pos) and sc_shape[0](site2.pos))

    def at_ll_interface(site1, site2):
        return ((ll_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and ll_shape[0](site2.pos)))

    def at_ml_interface(site1, site2):
        return ((ml_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and ml_shape[0](site2.pos)))

    def at_rl_interface(site1, site2):
        return ((rl_shape[0](site1.pos) and normal_shape[0](site2.pos)) or
                (normal_shape[0](site1.pos) and rl_shape[0](site2.pos)))

    # Replacements for various regions
    no_sc = "delta = 0"
    # Scaled hopping/onsites along x
    x_not_scaled = "x_scale = 1.0"
    # To kill hopping along y between N and SC
    y_not_scaled = "y_scale = 1.0"
    # Give hopping amplitudes to leads relative to normal one
    ll_interface = "T = T_ll"
    rl_interface = "T = T_rl"
    # The gate voltages should be provided in mV to have a corresponding
    # meV energy!
    gauss_pot = "V = -gauss_potential(site, V_gauss, dim_par)"
    no_pot = "V = 0"
    mu_sc = "mu = mu_sc"
    # Exclude the magnetic field from the superconductor
    no_mag_field = "Ez = 0"

    #### Construct the scattering region.
    # Central part of the scattering region.
    normal_shape = rectangle(dL, L+dL, dW_ML, dW_ML+W_L)
    # Following are parts of the scattering region.
    sc_shape = rectangle(dL, L+dL, dW_ML+W_L, dW_ML+W_L+dW_SC)
    ml_shape = rectangle(dL, L+dL, 0, dW_ML)
    if dL != 0:
        ll_shape = rectangle(0, dL, dW_ML, dW_ML+W_L)
        rl_shape = rectangle(L+dL, L+2*dL, dW_ML, dW_ML+W_L)

    # Onsite - the central normal part.
    onsite = tls.make_tb_system(bdg_ham,
        [no_sc, gauss_pot, x_not_scaled, y_not_scaled],
        pauli_matrices, dim=2)[0]
    sys[lat.shape(*normal_shape)] = onsite
    # Onsite - the superconducting part.
    # Scaled onsites/hoppings along x direction.
    onsite = tls.make_tb_system(bdg_ham,
        [mu_sc, gauss_pot, no_mag_field, y_not_scaled],
        pauli_matrices, dim=2)[0]
    sys[lat.shape(*sc_shape)] = onsite
    # Onsite - the normal lead parts.
    if dW_ML > 0:
        onsite = tls.make_tb_system(bdg_ham,
            [no_sc, gauss_pot, x_not_scaled, y_not_scaled],
            pauli_matrices, dim=2)[0]
        sys[lat.shape(*ml_shape)] = onsite
    if dL != 0:
        onsite = tls.make_tb_system(bdg_ham,
            [no_sc, gauss_pot, x_not_scaled, y_not_scaled],
            pauli_matrices, dim=2)[0]
        sys[lat.shape(*ll_shape)] = onsite
        onsite = tls.make_tb_system(bdg_ham,
            [no_sc, gauss_pot, x_not_scaled, y_not_scaled],
            pauli_matrices, dim=2)[0]
        sys[lat.shape(*rl_shape)] = onsite

    # Unchanged hoppings in entire scattering region.
    hops = tls.make_tb_system(bdg_ham, [x_not_scaled, y_not_scaled],
                              pauli_matrices, dim=2)[1]
    # Hoppings along x scaled in the superconducting part.
    sc_hops = tls.make_tb_system(bdg_ham, [y_not_scaled],
                                 pauli_matrices, dim=2)[1]
    # Modified hoppings at the junction between the normal part and the SC.
    sc_interface_hops = tls.make_tb_system(bdg_ham, [x_not_scaled],
                                           pauli_matrices, dim=2)[1]
    if dL != 0:
        ll_interface_hops = tls.make_tb_system(bdg_ham,
            [x_not_scaled, y_not_scaled, ll_interface],
            pauli_matrices, dim=2)[1]
        rl_interface_hops = tls.make_tb_system(bdg_ham,
            [x_not_scaled, y_not_scaled, rl_interface],
            pauli_matrices, dim=2)[1]

    for kind, value in hops.items():
        sys[kwant.builder.HoppingKind(kind, lat)] = value

    # Replace hoppings in the superconducting part of the scattering region.
    # Here we use scaled hoppings along x.
    for kind, value in sc_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if in_sc_scatter(i, j)
            )
        sys[hopping_iterator] = value

    # Hoppings between normal part and superconductor. These hoppings
    # are along y.
    for kind, value in sc_interface_hops.items():
        hopping_iterator = (
            (i, j)
            for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
            if at_sc_interface(i, j)
            )
        sys[hopping_iterator] = value

    if dL != 0:
        for kind, value in ll_interface_hops.items():
            hopping_iterator = (
                (i, j)
                for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
                if at_ll_interface(i, j)
                )
            sys[hopping_iterator] = value
        for kind, value in rl_interface_hops.items():
            hopping_iterator = (
                (i, j)
                for (i, j) in kwant.builder.HoppingKind(kind, lat)(sys)
                if at_rl_interface(i, j)
                )
            sys[hopping_iterator] = value

    #### Add leads.
    # Shape (width) of the leads
    def lr_lead_shape(pos):
        (x, y) = pos
        return (dW_ML <= y < dW_ML+W_L)

    # Normal leads (left and right)
    # Particle-hole degree of freedom is conserved.
    # Partition the scattering matrix into electron and hole blocks.
    PH_cons = np.kron(np.eye(2), -const.sz)
    left_lead = kwant.Builder(kwant.TranslationalSymmetry((-1, 0)),
                              conservation_law=PH_cons)
    right_lead = kwant.Builder(kwant.TranslationalSymmetry((1, 0)),
                               conservation_law=PH_cons)
    # Set chemical potential in left lead
    onsite, hops = tls.make_tb_system(bdg_ham,
        [no_sc, no_pot, x_not_scaled, y_not_scaled],
        pauli_matrices, dim=2)
    left_lead[lat.shape(lr_lead_shape, (-1, dW_ML))] = onsite
    # Set chemical potential in right lead
    onsite, hops = tls.make_tb_system(bdg_ham,
        [no_sc, no_pot, x_not_scaled, y_not_scaled],
        pauli_matrices, dim=2)
    right_lead[lat.shape(lr_lead_shape, (L+2*dL, dW_ML))] = onsite
    # Set the hoppings (same in both leads) and attach the leads
    for kind, value in hops.items():
        left_lead[kwant.builder.HoppingKind(kind, lat)] = value
        right_lead[kwant.builder.HoppingKind(kind, lat)] = value
    sys.attach_lead(left_lead)
    sys.attach_lead(right_lead)

    # Superconducting lead
    if not sc_chains:
        # We treat the superconductor as a single lead
        def middle_lead_shape(pos):
            (x, y) = pos
            return (dL <= x < L+dL)
        sc_lead = kwant.Builder(kwant.TranslationalSymmetry((0,1)))
        onsite, hops = tls.make_tb_system(bdg_ham,
            [mu_sc, no_mag_field, no_pot, y_not_scaled],
            pauli_matrices, dim=2)
        sc_lead[lat.shape(middle_lead_shape, (dL, dW_ML+W_L+dW_SC))] = onsite
        for kind, value in hops.items():
            sc_lead[kwant.builder.HoppingKind(kind, lat)] = value
        sys.attach_lead(sc_lead)
    else:
        # Treat the superconductor as multiple 1D leads attached in succession
        for i in range(dL, L+dL):
            def middle_lead_shape(pos):
                (x, y) = pos
                return x == i
            # Scaled hopping/onsites along x - here, it must be set to 0!
            zero_x_hopping = "x_scale = 0.0"
            sc_lead = kwant.Builder(kwant.TranslationalSymmetry((0,1)))
            onsite, hops = tls.make_tb_system(bdg_ham,
                [mu_sc, no_mag_field, no_pot, y_not_scaled, zero_x_hopping],
                pauli_matrices, dim=2)
            sc_lead[lat.shape(middle_lead_shape, (i, dW_ML+W_L+dW_SC))] = onsite
            for kind, value in hops.items():
                sc_lead[kwant.builder.HoppingKind(kind, lat)] = value
            sys.attach_lead(sc_lead)

    if plot_sys:
        kwant.plot(sys)
    sys = sys.finalized()
    return sys
